<?php

namespace Teknisa\CRM\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\CRM\Service\IntegracaoRDStationService;
use Teknisa\Libs\Util\ConnectionCustom;

class IntegracaoRDStationController  {
    
    protected $integracaoService;
    
    public function __construct (IntegracaoRDStationService $integracaoService, ConnectionCustom $connectionCustom) {
        $this->integracaoService = $integracaoService;
        $this->connectionCustom        = $connectionCustom;
    }
    
    /**
     * Recebe o redirecionamento do OAuth vindo do RDStation.
     * 
     * @param Request  $request
     * @param Response $response
     */
    public function auth(Request $request, Response $response) {
        try {
            $code = $_GET['code'];
            if($code){
                $refreshToken = $this->integracaoService->getToken($code);
                if($refreshToken){
                    $link = "<script>window.opener.postMessage({success:true, refreshToken:'$refreshToken'}, '*');window.close()</script>";
                    echo $link;    
                }
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveCustomField(Request $request, Response $response){
        try{
            $row = $request->getRow();
            
            $customField = $this->integracaoService->saveCustomField($row);

            $response->addDataSet(new DataSet("CUSTOM_FIELD", array("uuid" => $customField["uuid"])));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importLead(){
        try{
            if(isset($_GET['organization'])){
                /*Pega o lead passado pelo RDStation*/
                $lead = file_get_contents('php://input');
                
                /* Teste Fixo  */
                //$lead = '{"leads":[{"id":"47739094","email":"alexg.melo@bol.com.br","name":"Alex","company":"Empreitec","job_title":"Sócio","bio":null,"public_url":"http://rdstation.com.br/leads/public/76b011ba-3ee4-40b4-ad52-21decf3075bf","created_at":"2015-11-03T09:33:56.345-02:00","opportunity":"false","number_conversions":"1","user":null,"first_conversion":{"content":{"identificador":"ebook-5-maneiras-simples-de-dirigir-a-sua-empresa","traffic_source":null,"email_lead":"alexg.melo@bol.com.br","nome":"Alex","empresa":"Empreitec","cargo":"Sócio","website":"","telefone":""},"created_at":"2015-11-03T09:33:57.036-02:00","cumulative_sum":"1","source":"ebook-5-maneiras-simples-de-dirigir-a-sua-empresa","conversion_origin":{"source":"odhen.com","medium":"referral","value":"","campaign":"(referral)","channel":"Referral"}},"last_conversion":{"content":{"identificador":"ebook-5-maneiras-simples-de-dirigir-a-sua-empresa","traffic_source":null,"email_lead":"alexg.melo@bol.com.br","nome":"Alex","empresa":"Empreitec","cargo":"Sócio","website":"","telefone":""},"created_at":"2015-11-03T09:33:57.036-02:00","cumulative_sum":"1","source":"ebook-5-maneiras-simples-de-dirigir-a-sua-empresa","conversion_origin":{"source":"odhen.com","medium":"referral","value":"","campaign":"(referral)","channel":"Referral"}},"custom_fields":{},"website":null,"personal_phone":null,"mobile_phone":null,"city":null,"state":null,"tags":null,"lead_stage":"Lead","last_marked_opportunity_date":null,"uuid":"76b011ba-3ee4-40b4-ad52-21decf3075bf","fit_score":"d","interest":15}]}';
                /* Teste Dinâmico */
                //file_put_contents('/home/developer/workfolder/tmp/teste', $lead);
                
                $lead = json_decode($lead, true);
                $org  = $_GET['organization'];
                
                /*Conecta com a organização passada como parâmetro*/
                $entityManager = $this->connectionCustom->updateVpdConnection($org);
                
                /*Sobrescreve o entityManager usado pela NovoCodigo*/
                \Setforms2\Shared\Registry::store("EntityManager", $entityManager, true);
                \Setforms2\Shared\Registry::store("EntityManagerConnection", $entityManager->getConnection(), true);
                
                if($entityManager){
                    $this->integracaoService->importLead($lead['leads'][0], $entityManager, $org);
                }
                
            }
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function createOrConvertLead(Request $request, Response $response){
        try{
            $row = $request->getRow();
            
            $lead = $this->integracaoService->createOrConvertLead($row);
            
            $response->addDataSet(new DataSet("LEAD", array("uuid" => $customField["uuid"])));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteCustomFields(Request $request, Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            
            $fieldsDeleted = [];
            foreach($rows as $row){
                $fieldsDeleted[] = $this->integracaoService->deleteCustomField($row);
            }
            
            $response->addDataSet(new DataSet("DELETED_FIELDS", array("count" => count($fieldsDeleted))));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function sendContactToRD(Request $request, Response $response){
        try{
            $row = $request->getRow();
            
            $contact = $this->integracaoService->sendContactToRD($row);
            if(isset($contact['result']) && $contact['result'] === 'success'){
                $this->changeFunnel($request, $response);
            }
            $response->addDataSet(new DataSet("CONTACT", array("created" => $contact)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function changeFunnel(Request $request, Response $response){
        try{
            $row = $request->getRow();
            /*API Antiga usa números
            $row['FUNNEL'] = ($row['FUNNEL'] === 'Lead') ? 0 : (($row['FUNNEL'] === 'Qualified Lead') ? 1 : 2);
            */
            
            $funnel = $this->integracaoService->changeFunnel($row);
            
            $response->addDataSet(new DataSet("FUNNEL", array("changed" => $funnel)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function markSale(Request $request, Response $response){
        try{
            $row = $request->getRow();
            $sale = $this->integracaoService->markSale($row);
            
            $response->addDataSet(new DataSet("SALE", array("mark" => $sale)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function feedBackConversion(Request $request, Response $response){
        try{
            $row = $request->getRow();
            $conversion = $this->integracaoService->feedBackConversion(array('EVENT_NAME' => $row['EVENT_NAME'], 'EMAIL' => $row['EMAIL']));
            
            $response->addDataSet(new DataSet("CONVERSION", array("conversion" => $conversion)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function markOpport(Request $request, Response $response){
        try{
            $row = $request->getRow();
            
            $opport = $this->integracaoService->markOpport($row['EMAIL']);
            
            $response->addDataSet(new DataSet("OPPORT", array("marked" => $opport)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
}

?>