<?php
namespace Controller;

use Service\PerfacessparcOperadorRepository;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use ZeedhiUser\Controllers\AccessControl\AccessControlController;
use ZeedhiUser\Controllers\AccessControl\AccessControlRule;
use ZeedhiUser\Controllers\General\ZhuCrud;
use ZeedhiUser\Services\AccessControl\Environment;

class PerfacessparcOperador extends ZhuCrud {

    protected $dataSourceName = "perfacessparcOperador";
    protected $dataSourceNameToSave = "perfacessparc";
    protected $key = 1;

    protected $perfacessparcOperadorRepository;

    public function __construct(Manager $dataSourceManager, Environment $environment, ParameterBag $parameterBag, PerfacessparcOperadorRepository $perfacessparcOperadorRepository) {
        $this->perfacessparcOperadorRepository = $perfacessparcOperadorRepository;
        parent::__construct($dataSourceManager, $environment, $parameterBag);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $index = 1;
        try {
            $maxNRPERFACESSPARC = $this->perfacessparcOperadorRepository->getMaxNrPerfAcessParc();
        }
        catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
            $maxNRPERFACESSPARC = 1;
        }

        foreach($request->getDataSet()->getRows() as $row) {
            if ($row->get("__is_new")) {
                $row->set("NRPERFACESSPARC", $maxNRPERFACESSPARC+$index);
                $index++;
            }
            $row->set("DTVALPERFILPARC", null);
            $this->setAuditingFields($row);
        }

        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceNameToSave);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} row(s) persisted with success", DTO\Response\Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function setAuditingFields (&$row) {
        $row->set("NRORG", $this->environment->getOrganizationId());
        $row->set("IDATIVO", "S");
        $row->set("DTINCLUSAO", $this->environment->getCurrentTime()->format("d/m/Y H:i:s"));
        $row->set("DTULTATU", $this->environment->getCurrentTime()->format("d/m/Y H:i:s"));
        $row->set("NRORGINCLUSAO", $this->environment->getOrganizationId());
        $row->set("NRORGULTATU", $this->environment->getOrganizationId());
        $row->set("CDOPERINCLUSAO", $this->environment->getOrganizationId());
        $row->set("CDOPERULTATU", $this->environment->getOrganizationId());
    }

}