<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Teknisa\CRM\Service\GeralService;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;

class crm00050000RevendaController extends TeknisaCrud
{
    protected $dataSourceName = 'crm_revenda';
    
    protected $revendaService;
    protected $newCode;
    protected $entityManager;
    protected $connection;
    
    public function __construct ($crudService, $parameterBag, $entityManager,$revendaService, $newCode){
        
        $this->newCode          = $newCode;
        $this->revendaService   = $revendaService;
        $this->entityManager    = $entityManager;
        $this->connection       = $entityManager->getConnection();
        
        parent::__construct($crudService, $parameterBag);
        
    }
    
    public function findCliente(Request $request, Response $response){
        $this->dataSourceName = 'cliente_comunicaparc';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function save(Request $request, Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $this->updateComunicaRevenda($request);
        parent::save($request, $response);
        
	    $columnsToReturn = $this->crudService->getColumnsToReturn();
	    
        $retorno = array();
        if (count($columnsToReturn) > 0)
            $retorno["NRREVENDA"] = $columnsToReturn[0]["NRREVENDA"];
	    
	    $response->addDataSet(new DataSet("columnsToReturn", $retorno));
    }
    
    public function deleteRevenda($request, $response)
    {
        parent::delete($request, $response);
        
        /*Deleta Parceria do tipo Revenda do cliente relacionado a revenda*/
        $row = $request->getRow();
        
        $deleteParceria = "DELETE PARCERIA
                    WHERE CDTIPOPARCEIRO = :CDTIPOPARCEIRO
                      AND NRPARCNEGOCIO = :NRPARCNEGOCIO
                      AND NRORG            = :NRORG";

        $params = array
        (
            "CDTIPOPARCEIRO"    => 'REVENDA',
            "NRPARCNEGOCIO"     => $row["NRPARCNEGOCIO"],
            "NRORG"             => $row["NRORG"]
        );

        try
        {
            $std = $this->connection->prepare($deleteParceria);
            $std->execute($params);
        }catch (\Exception $e)
        {
            throw $e;
        }
    }
    
    public function createNrParcNegocioAndInsertParcNegocio($row, $response){
        
        $row = $row->getRow();
        
        $row["IDPESSOAFISICA"] = self::isPhysicalPerson($row["NRINSCRIREVENDA"]); 
       
        if($row['__newClient']){
            
            $row["NRPARCNEGOCIO"] = $this->newCode->getCodeByTableName("PARCNEGOCIO", $row["NRORG"]);
            
            $paramsInsertParcNegocio = array(
                "IDATIVO"               => $row["IDATIVO"],
                "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                "NRORG"                 => $row["NRORG"],
                "NMPRINCIPALPARC"       => $row["NMRAZSOCREVENDA"],
                "NMSECUNDARIPARC"       => $row["NMFANTREVENDA"],
                "NRINSCRICAOPARC"       => $row["NRINSCRIREVENDA"],
                "NRORGINCLUSAO"         => $row["NRORG"],
                "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"],
                "CDTIPOPARCPRINCIPAL"   => "REVENDA",
                "CDTIPOINSCRICAO"       => $row["CDTIPOINSCRICAO"],
                "IDPESSOAFISICA"        => $row["IDPESSOAFISICA"],
            );
            
            $this->revendaService->insertParcNegocio($paramsInsertParcNegocio);
            
            $paramsInsertParceria = array(
                "IDATIVO"               => $row["IDATIVO"],
                "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                "NRORG"                 => $row["NRORG"],
                "NRPARCERIA"            => $this->newCode->getCodeByTableName("PARCERIA", $row["NRORG"]),
                "CDTIPOPARCEIRO"        => "REVENDA",
                "NRORGINCLUSAO"         => $row["NRORG"],
                "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
             );
             
            $this->revendaService->insertParceria($paramsInsertParceria);
            
            
        }else{
            $row["NRPARCNEGOCIO"] = $this->revendaService->getParcByClient($row['CDCLIENTE'], $row['NRORG']);
            $paramsInsertParceria = array(
                "IDATIVO"               => $row["IDATIVO"],
                "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                "NRORG"                 => $row["NRORG"],
                "NRPARCERIA"            => $this->newCode->getCodeByTableName("PARCERIA", $row["NRORG"]),
                "CDTIPOPARCEIRO"        => "REVENDA",
                "NRORGINCLUSAO"         => $row["NRORG"],
                "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
             );
            $this->revendaService->insertParceria($paramsInsertParceria);
            
        }   
        if(isset($row['ENDERECO'])){
            $endereco = $row['ENDERECO'];
            $endereco['NRENDERECOPARC'] = $this->newCode->getCodeByTableName('ENDERECOPARC', $row['NRORG'], 1, 6);
            $paramsInsertEnderecoparc = array(
                    "IDATIVO"               => $row["IDATIVO"],
                    "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                    "NRORG"                 => $row["NRORG"],
                    "NRENDERECOPARC"        => $endereco['NRENDERECOPARC'],
                    "CDTIPOENDERECO"        => $endereco['CDTIPOENDERECO'],
                    "NRORGINCLUSAO"         => $row["NRORG"],
                    "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"],
                    "CDPAIS"                => $endereco["CDPAIS"],
                    "SGESTADO"              => $endereco["SGESTADO"],
                    "CDMUNICIPIO"           => $endereco["CDMUNICIPIO"],
                    "CDBAIRRO"              => $endereco["CDBAIRRO"],
                    "NMBAIRROENDERECO"      => $endereco["NMBAIRRO"],
                    "DSCOMPLEENDERECO"      => $endereco["DSCOMPLEENDERECO"],
                    "NRCEPENDERECO"         => $endereco["NRCEPENDERECO"],
                    "DSENDERECO"            => $endereco["DSENDERECO"],
                    "NRIMOVELENDERECO"      => $endereco["NRIMOVELENDERECO"]
                 );
                 
            $this->revendaService->insertEndeparc($paramsInsertEnderecoparc);
        }
        
        if(isset($row['COMUNICAPARC'])){
            $comunica = $row['COMUNICAPARC'];
            foreach($comunica as $comunicaparc){
                $comunica['NRCOMUNICAPARC'] = $this->newCode->getCodeByTableName('COMUNICAPARC', $row['NRORG'], 1, 6);
                $paramsInsertComunicaparc = array(
                    "IDATIVO"               => $row["IDATIVO"],
                    "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
                    "NRORG"                 => $row["NRORG"],
                    "NRCOMUNICAPARC"        => $comunica['NRCOMUNICAPARC'],
                    "CDCOMUNICAPARC"        => $comunicaparc['value'],
                    "CDFORMACOMU"           => $comunicaparc['name'] === 'NRTELREVENDA' ? '02' : '05',
                    "NRORGINCLUSAO"         => $row["NRORG"],
                    "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
                 );
                $this->revendaService->insertComunicaparc($paramsInsertComunicaparc);
            }  
        }
        
        
        if(isset($endereco) && isset($comunica)){
            $dataSetResponse = new DataSet("DADOS",array("NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"], "NRENDERECOPARC" => $endereco['NRENDERECOPARC'], "NRCOMUNICAPARC" => $comunica['NRCOMUNICAPARC']));
        }else if(isset($endereco)){
            $dataSetResponse = new DataSet("DADOS",array("NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"], "NRENDERECOPARC" => $endereco['NRENDERECOPARC']));
        }else if(isset($comunica)){
            $dataSetResponse = new DataSet("DADOS",array("NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"], "NRCOMUNICAPARC" => $comunica['NRCOMUNICAPARC']));
        }else{
            $dataSetResponse = new DataSet("DADOS",array("NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"]));
        }
        
        $response->addDataSet($dataSetResponse);
        
    }
    
    public function insertParcInterloc($row, $response){
        
        $row = $row->getRow();
        $row["NRPARCNEGOCIO"] = $this->newCode->getCodeByTableName("PARCNEGOCIO", $row["NRORG"]);
        
        $paramsInsertParcNegocio = array(
            "IDATIVO"               => $row["IDATIVO"],
            "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIO"],
            "NRORG"                 => $row["NRORG"],
            "NMPRINCIPALPARC"       => $row["NMPRINCIPALPARC"],
            "NMSECUNDARIPARC"       => $row["NMSECUNDARIPARC"],
            "NRINSCRICAOPARC"       => $row["NRINSCRICAOPARC"],
            "NRORGINCLUSAO"         => $row["NRORG"],
            "CDOPERINCLUSAO"        => $row["CDOPERADOR"],
            "CDTIPOPARCPRINCIPAL"   => "INTERLOCUTOR",
            "CDTIPOINSCRICAO"       => $row["CDTIPOINSCRICAO"],
            "IDPESSOAFISICA"        => $row["IDPESSOAFISICA"],
        );
        
        $this->revendaService->insertParcNegocio($paramsInsertParcNegocio);
        $dataSetResponse = new DataSet("DADOS",array("NRPARCNEGOCIOINTERLOCUTOR" => $row["NRPARCNEGOCIO"]));
        
        $response->addDataSet($dataSetResponse);
        
    }
    
     public function insertResaleInterloc($row, $response){
        
        $row = $row->getRow();
        $row["NRSEQINTERLOCUTOR"] = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $row["NRORG"]);
        
        $paramsInsertResaleInterloc = array(
            "NRSEQINTERLOCUTOR"     => $row["NRSEQINTERLOCUTOR"],
            "NMINTERLOCUTOR"        => $row["INTERLOCUTOR"],
            "NRPARCNEGOCIO"         => $row["NRPARCNEGOCIOREV"],
            "IDATIVO"               => 'S',
            "NRORG"                 => $row["NRORG"],
            "NRTELEFONEINTERLOC"    => $row["NRTELEFONEINTERLOC"],
            "EMAILINTERLOC"         => $row["DSEMAILINTERLOC"],
            "CDOPERADOR"            => $row["CDOPERADOR"]
        );
        
        $this->revendaService->insertResaleInterloc($paramsInsertResaleInterloc);
        
    }
    
    public function replicaRevenda($row, $response){
        
        if(isset($row['COMUNICAPARC']) ){
            foreach($row['COMUNICAPARC'] as $comunicaparc){
                $paramsInsertComunicaparc = array(
                    "IDATIVO"               => $row["IDATIVO"],
                    "NRREVENDA"             => $row["NRREVENDA"],
                    "NRCOMUNICAPARC"        => $row['NRCOMUNICAPARC'],
                    "NRORG"                 => $row["NRORG"],
                    "NRCOMUNICAREVENDA"     => $this->newCode->getCodeByTableName('CRM_COMUNICAREVENDA', $row['NRORG'], 1, 6),
                    "CDCOMUNICACAO"         => $comunicaparc['value'],
                    "CDFORMACOMU"           => $comunicaparc['name'] === 'NRTELREVENDA' ? '02' : '05',
                    "NRORGINCLUSAO"         => $row["NRORG"],
                    "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
                 );
                $this->revendaService->insertComunicaRevenda($paramsInsertComunicaparc);
            }
        }else{
            if(!$row['__newClient']){
                $comunica = $this->revendaService->getComunicaByClient($row['CDCLIENTE'], $row['NRORG']);
                if($comunica){
                    foreach($comunica as $comunicaparc){
                         $paramsInsertComunicaparc = array(
                            "IDATIVO"               => $row["IDATIVO"],
                            "NRREVENDA"             => $row["NRREVENDA"],
                            "NRCOMUNICAPARC"        => $comunicaparc['NRCOMUNICAPARC'],
                            "NRORG"                 => $row["NRORG"],
                            "NRCOMUNICAREVENDA"     => $this->newCode->getCodeByTableName('CRM_COMUNICAREVENDA', $row['NRORG'], 1, 6),
                            "CDCOMUNICACAO"         => $comunicaparc['CDCOMUNICAPARC'],
                            "CDFORMACOMU"           => $comunicaparc['CDFORMACOMU'],
                            "NRORGINCLUSAO"         => $row["NRORG"],
                            "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
                         );
                        $this->revendaService->insertComunicaRevenda($paramsInsertComunicaparc);
                    }
                }   
            }
        }
    }
    
    public function isPhysicalPerson($nrInscription){
    
        if(count($nrInscription) == 11){
            
            return "S";
                
        }else{
            
            return "N";
            
        }
        
    }
    
    public function validaCliente($row, $response){
        try{
            $row = $row->getRow();
            
            $clientes = $this->revendaService->validaCliente($row);
            $response->addDataSet(new DataSet("DADOS", array("CLIENTES" => $clientes)));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function updateComunicaRevenda($request){
        //Faz update da tabela CRM_COMUNICA_REVENDA baseado na edição nos campos de comunicação da revenda
        foreach($request->getDataSet()->getRows() as $row) {
            if(!$row['__is_new']){
                if(isset($row['OLDCOMUNICAPARC'])){
                    foreach($row['OLDCOMUNICAPARC'] as $comunicaparc){
                        //Pega o NRCOMUNICAREVENDA da comunicação 'antiga'
                        $oldComunica = $this->revendaService->verifyComunicaRevenda($comunicaparc, $row["NRORG"], $row["NRREVENDA"]);
                        if(!empty($oldComunica)){
                            //Pega o valor da 'nova' comunicação
                            foreach($row['COMUNICAPARC'] as $update){
                                if($update['name'] === $comunicaparc['name']){
                                    $value = $update['value'];
                                }
                            }
                            $paramsUpdateComunicaparc = array(
                                    "NRREVENDA"             => $row["NRREVENDA"],
                                    "CDCOMUNICACAO"         => $value,
                                    "NRORG"                 => $row["NRORG"],
                                    "NRCOMUNICAREVENDA"     => $oldComunica[0]['NRCOMUNICAREVENDA']
                            );
                            $this->revendaService->updateComunicaRevenda($paramsUpdateComunicaparc);    
                        }else{
                            //Se não houver comunicação com aqueles dados, o insert na tabela crm_comunicarevenda é efetuado
                            $comunica['NRCOMUNICAPARC'] = $this->newCode->getCodeByTableName('COMUNICAPARC', $row['NRORG'], 1, 6);
                            $paramsInsertComunicaparc = array(
                                "IDATIVO"               => $row["IDATIVO"],
                                "NRREVENDA"             => $row["NRREVENDA"],
                                "NRCOMUNICAPARC"        => $comunica['NRCOMUNICAPARC'],
                                "NRORG"                 => $row["NRORG"],
                                "NRCOMUNICAREVENDA"     => $this->newCode->getCodeByTableName('CRM_COMUNICAREVENDA', $row['NRORG'], 1, 6),
                                "CDCOMUNICACAO"         => $comunicaparc['value'],
                                "CDFORMACOMU"           => $comunicaparc['name'] === 'NRTELREVENDA' ? '02' : '05',
                                "NRORGINCLUSAO"         => $row["NRORG"],
                                "CDOPERINCLUSAO"        => $row["CDOPERREVENDA"]
                             );
                            $this->revendaService->insertComunicaRevenda($paramsInsertComunicaparc);
                        }
                    }
                }
            }
        }
    }
    
    public function getEndeClie($row, $response){
        try{
            $row = $row->getRow();
            
            $endereco = $this->revendaService->getEndeClie($row['CDCLIENTE'], $row['NRORG']);
            if(is_array($endereco)){
                $possuiEndereco = true;
            }else{
                $possuiEndereco = false;
            }
            $response->addDataSet(new DataSet("DADOS", array("POSSUIENDERECO" => $possuiEndereco)));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

}
?>