<?php
namespace Teknisa\CRM\Service;


class IntegracaoRDStationService {
    protected $importacaoFactory;
    const URL_INTEGRACAO = 'https://api.rd.services';
    const UNAUTHORIZED   = 'UNAUTHORIZED';
    const FIRST_TIME     = 'FIRST_TIME';
    const REFRESH        = 'REFRESH';
    const MISSING_AUTH   = 'Usuário não autenticado no RDStation!';
    const CLIENT_ID      = '2de5e1c4-b917-4d0b-9356-7a67c6a89362';
    const CLIENT_SECRET  = '4QNhLB3ch4XPhM7D0svXHJRQrsXdLDjW';
    
    public function __construct ($importacaoFactory, $integracaoFactory, $session){
        $this->importacaoFactory   = $importacaoFactory;
        $this->integracaoFactory   = $integracaoFactory;
        $this->session             = $session;
    }
    
    /**
     * Executa uma requisição curl.
     * 
     * @param string  $token
     * @param string  $route
     * @param mixed[] $requestParam
     * 
     * @return mixed[]
     */
    private function executeCurl($token, $route, $requestParam, $method = null) {
        set_time_limit(180);
        $header = array();
        $header[] = "Authorization: Bearer " . $token;
        $header[] = "Content-Type: application/json";

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
        
        if(strpos($route, 'https') !== false){
            curl_setopt($curl, CURLOPT_URL, $route);    
        }else{
            curl_setopt($curl, CURLOPT_URL, self::URL_INTEGRACAO . $route);
        }
        
        if($method === null)
            curl_setopt($curl, CURLOPT_POST, true);
        else
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        
        if($requestParam !== null)
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($requestParam));
            
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        $result = curl_exec($curl);
        $result = json_decode($result, true);
        $error = curl_error($curl);
        curl_close($curl);
        
        if (!empty($result["errors"])) {
            //Encode necessário pois o RD retorno um objeto com os erros
            throw new \Exception(json_encode($result["errors"]));
        }
        else if (!empty($error)) {
            throw new \Exception($error);
        }
        else {
            return $result;
        }
    }
    
    /**
     * Retorna o token para a comunicação com o RDStation.
     * 
     * @return string
     */
    public function getToken($code, $mode = self::FIRST_TIME) {
        $token = $this->session->get("rdstation_token");
        $token = null;
        if ($token)
            return $token;
        else {
            try{
                
                $client_id = self::CLIENT_ID;
                $client_secret = self::CLIENT_SECRET;
                
                if($mode === self::REFRESH){
                    $requestParam = array(
                        "client_id"     => $client_id,
                        "client_secret" => $client_secret,
                        "refresh_token" => $code
                    );    
                }else{
                    $requestParam = array(
                        "client_id"     => $client_id,
                        "client_secret" => $client_secret,
                        "code"          => $code
                    );
                }
                $token = $this->executeCurl(null, "/auth/token", $requestParam);
                $refreshToken = $token["refresh_token"];
                $token = $token["access_token"];
                $this->session->set("rdstation_token", $token);
                $this->session->set("rdstation_refreshToken", $refreshToken);
                $this->integracaoFactory->saveRefreshToken($refreshToken);
                
                return $refreshToken;
            }catch(\Exception $e){
                throw new \Exception(json_encode($e->getMessage()));
            }
        }
    }
    
    public function saveCustomField($row){
        try{
                $token = $this->session->get("rdstation_token");
                if(!$token){
                    $e = '{error_type:'.self::MISSING_AUTH.'}';
                    throw new \Exception($e, 401);
                }
                $field = array(
                    "name"  => array(
                        "pt-BR" => $row['NMCAMPORDSTATION']
                    ),
                    "label" => array(
                        "pt-BR" => substr($row['DSCAMPORDSTATION'],0,999)
                    ),
                    "api_identifier" => substr('cf_'.strtolower(strtolower($row['NMCAMPORDSTATION'])), 0, 63),
                    "data_type" => "",
                    "presentation_type" => "",
                    "validation_rules" => ""
                );
                $this->buildCustomField($field, $row);
                
                if(isset($row['IDUUID']) && !empty($row['IDUUID'])){
                    unset($field['api_identifier']);
                    $field = $this->executeCurl($token, "/platform/contacts/fields/".$row['IDUUID'], $field, 'PATCH');    
                }else{
                    $field = $this->executeCurl($token, "/platform/contacts/fields", $field);
                }
                return $field;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'saveCustomField', $row);
            }
    }
    
    public function deleteCustomField($row){
        try{
                $token = $this->session->get("rdstation_token");
                if(!$token){
                    $e = '{error_type:'.self::MISSING_AUTH.'}';
                    throw new \Exception($e, 401);
                }
                $uuid = $row['IDUUID'];
                $this->executeCurl($token, "/platform/contacts/fields/$uuid", null, 'DELETE');
                
                return $uuid;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'deleteCustomField', $row);
            }
    }
    
    public function importLead($lead, $entityManager, $org){
        try{
            $this->entityManager = $entityManager;
            $this->org           = $org;
            
            $connection = $this->entityManager->getConnection();
            
            $connection->beginTransaction();
            
            $formatedLead = $this->buildData($lead);
            if($this->checkRulesToImportLead($lead)){
                
                $this->importacaoFactory->setConnectionAndDataSet($formatedLead, $this->entityManager, $this->org);
                //Se o lead já existe evolui, se não cria.
                if($this->searchLead($formatedLead)){
                    //Se o parâmetro de Gerar tarefa for 'Sim', verifica se já existe tarefa em aberto, se existir não duplica.
                    if($formatedLead['IDGERATAREFA'] === 'S')
                        $this->checkNeedNewTask($formatedLead);
                        
                    $this->importacaoFactory->evoluiLeadRDStation($formatedLead);
                    $feedBackEvent = 'Cadastro evoluido no CRM';
                }else{
                    $nrparcnegocio = $this->importacaoFactory->importLeadRDStation();
                    $this->setCustomFields($nrparcnegocio, $lead['custom_fields']);
                    $feedBackEvent = 'Cadastro realizado no CRM';
                }
                
                $this->feedBackConversion(array('EVENT_NAME' => $feedBackEvent, 'EMAIL' => $formatedLead['EMAIL']));
            }
            
            $connection->commit();
            
        }catch(\Exception $e){
            $connection->rollBack();
            
            throw new \Exception($e);
        }
    }
    
    private function buildData($lead){
        return array(
            'CNPJ'                         => '99999999999999',
            'RAZAO_SOCIAL'                 => $lead['company'],
            'NOME_FANTASIA'                => $lead['company'],
            'NR_FUNCIONARIOS'              => isset($lead['last_conversion']['content']['company_attributes']['address']) ? $lead['last_conversion']['content']['company_attributes']['address'] : '',
            'TIPODEPARCEIRO'               => 'CANDIDATO',
            'ATIVO'                        => 'S',
            'COMO_CHEGOU_ATE_NOS'          => $this->getCdmododivulgRdstation(),
            'COD_SEGMENTO'                 => null,
            'CDOPERRESPONSAVEL'            => $this->getOperRdstation($lead['user'])['CDOPERADOR'],
            'NRPARCRESPONSAVEL'            => $this->getOperRdstation($lead['user'])['NRPARCNEGOCIO'],
            'CEP'                          => null,
            'UF__ESTADO'                   => $lead['state'],
            'COD_MUNICIPIO'                => $lead['city'],
            'BAIRRO'                       => null,
            'ENDERECO'                     => isset($lead['last_conversion']['content']['company_attributes']['address']) ? $lead['last_conversion']['content']['company_attributes']['address'] : '',
            'NUMERO'                       => null,
            'COMPLEMENTO'                  => null,
            'TELEFONE'                     => isset($lead['last_conversion']['content']['company_attributes']['phone']) ? $lead['last_conversion']['content']['company_attributes']['phone'] : '',
            'TELEFONE_2'                   => null,
            'EMAIL'                        => $lead['email'],
            'IDSTATUSCAN'                  => $lead['opportunity'] === "true" ? '3' : '4',
            'NOME_INTERLOCUTOR'            => $lead['name'],
            'CARGO'                        => $lead['job_title'],
            'CEP__INTERLOCUTOR'            => null,
            'UF__ESTADO__INTERLOCUTOR'     => $lead['state'],
            'COD_MUNICIPIO__INTERLOCUTOR'  => $lead['city'],
            'BAIRRO__INTERLOCUTOR'         => null,
            'ENDERECO__INTERLOCUTOR'       => isset($lead['last_conversion']['content']['company_attributes']['address']) ? $lead['last_conversion']['content']['company_attributes']['address'] : '',
            'NUMERO__INTERLOCUTOR'         => null,
            'COMPLEMENTO__INTERLOCUTOR'    => null,
            'TELEFONE__INTERLOCUTOR'       => $lead['personal_phone'],
            'EMAIL__INTERLOCUTOR'          => $lead['email'],
            'RECEBE_PRESENTE'              => 'N',
            'NOME_DO_RESPONSAVEL_GERAL'    => null,
            'NOME_DO_PONTO_DE_ATENDIMENTO' => null,
            'NUMERO_DE_INTEGRACAO'         => null,
            'IDGERATAREFA'                 => $this->getCreateTaskParam(),
            'COD_OPERADOR_TAREFA'          => $this->getOperRdstation($lead['user'])['CDOPERADOR'],
            'DSFILACONTATO'                => 'Gerado automaticamente pela integração com o RDStation',
            'DTPLANFILACONTA'              => $this->getNextDay(),
            'NRPRIORFILACONT'              => '',
            'IDGERAEVOLUCAO'               => 'S',
            'DTHREVOLCONT'                 => $this->buildDate($lead['last_conversion']['created_at']),
            'CDTIPOCONT'                   => null,
            'CDMOTIEVOCON'                 => null,
            'NRSEQRESULEVOL'               => null,
            'TXEVOLCONT'                   => 'Gerado automaticamente pela integração com o RDStation. 
Gerado pelo evento: '.$lead['last_conversion']['source'],
            'SENTIDO'                      => 'E'
        );
    }
    
    private function getCdmododivulgRdstation(){
        return $this->integracaoFactory->getCdmododivulgRdstation($this->entityManager, $this->org);
    }
    
    private function getCreateTaskParam(){
        $param = $this->integracaoFactory->getCreateTaskParam($this->entityManager, $this->org);
        
        if(empty($param) || $param == 'N'){
            return 'N';
        }else{
            return 'S';
        }
    }
    
    private function getOperRdstation($owner = null){
        $oper = null;
        //Se o lead do RD tiver um dono, busca o operador dele no CRM
        if($owner !== null)
            $oper = $this->integracaoFactory->getOwnerOper($this->entityManager, $this->org, $owner);
        
        if(empty($oper))        
            $oper = $this->integracaoFactory->getOperRdstation($this->entityManager, $this->org);
        
        if(empty($oper)){
            return array(
                'CDOPERADOR'    => null,
                'NRPARCNEGOCIO' => null
            );
        }else{
            return $oper;
        }
    }
    
    private function getNextDay(){
        $date = new \DateTime();
        $date = $date->add(new \DateInterval('P1D'));
        return $date->format('d/m/Y H:i');
    }
    
    private function searchLead(&$formatedLead){
        $lead = $this->integracaoFactory->searchLead($this->entityManager, $this->org, $formatedLead['EMAIL']);
        if(!empty($lead)){
            $formatedLead['NRSEQCONTATO'] = $lead['NRSEQCONTATO'];
            $formatedLead['NRPARCNEGOCIO'] = $lead['NRPARCNEGOCIO'];
            return true;
        }else{
            return false;
        }
    }
    
    /**
     * Converte a data vinda do RD para dd/mm/yyyy
     * 
     * Formato padrão do RD: yyyy-mm-dd
     *          2018-04-26T10:48:58.675-03:00"
    */
    private function buildDate($dateRd){
        $year  = substr($dateRd, 0, 4);
        $month = substr($dateRd, 5, 2);
        $day   = substr($dateRd, 8, 2);
        
        return ($day.'/'.$month.'/'.$year);
    }
    
    
    private function checkRulesToImportLead($lead){
        
        //Verifica Eventos de Conversão
        $eventsToImport = $this->integracaoFactory->getEventsRDParam($this->entityManager, $this->org);
        $leadEvent = mb_strtoupper($lead['last_conversion']['source']);
        if(strpos($eventsToImport, $leadEvent) === false && !empty($eventsToImport)){
            return false;
        }
        
        //Verifica Domínios de email ignorados
        $domainsToIgnore = $this->integracaoFactory->getDomainsRDParam($this->entityManager, $this->org);
        $interlocEmail = strtoupper($lead['email']);
        $domain = substr($interlocEmail, strpos($interlocEmail, '@'));
        if(strpos($domainsToIgnore, $domain) !== false){
            return false;
        }
        
        //Verifica Cargos dos Interlocutores
        $jobsToImport = $this->integracaoFactory->getJobTitleRDParam($this->entityManager, $this->org);
        $jobsToImport = explode(',',$jobsToImport);
        $interlocJobTitle = strtoupper(\Teknisa\CRM\Util\Util::removeCaracteresEspeciais($lead['job_title']));
        
        $import = false;
        foreach($jobsToImport as $job){
            //Remove os espaços
            $interlocJobTitle = str_replace(" ", "", $interlocJobTitle);
            $job = str_replace(" ", "", \Teknisa\CRM\Util\Util::removeCaracteresEspeciais($job));
            
            if($interlocJobTitle === $job){
                $import = true;
            }
            //Procura os cargos permitidos no cargo do interlocutor(Ex.: Parametrização = 'Diretor', Interlocutor = 'Diretor de Marketing')
            if(!empty($job) && strpos($interlocJobTitle, $job) !== false){
                $import = true;
            }
            /*
            //Procura o cargo do interlocutor nos cargos permitidos(Ex.: Parametrização = 'Diretor de Marketing', Interlocutor = 'Diretor')
            if(strpos($job, $interlocJobTitle) !== false){
                $import = true;
            }
            */
        }
        if(!$import && !empty($jobsToImport[0])){
            return false;
        }
        //Verifica o score dos leads vindos do RDStation
        $scoresToImport = $this->integracaoFactory->getScoresRDParam($this->entityManager, $this->org);
        $scoresToImport = explode(',',$scoresToImport);
        $leadScore = strtoupper($lead['fit_score']);
        
        $import = false;
        foreach($scoresToImport as $score){
            if($score === $leadScore){
                $import = true;
            }    
        }
        if(!$import && !empty($scoresToImport[0])){
            return false;
        }
        
        
        return true;
    }
    
    /**
     * Verifica se há tarefas em aberto para o parceiro
     * daquele lead, se houver muda o parâmetro para não
     * duplicar tarefas em aberto.
    */
    private function checkNeedNewTask(&$formatedLead){
        $tasksOpen = $this->integracaoFactory->getTasksOpen($this->entityManager, $this->org, $formatedLead['NRPARCNEGOCIO']);
        if(!empty($tasksOpen)){
            $formatedLead['IDGERATAREFA'] = 'N';
        }
    }
    
    private function buildCustomField(&$customField, $row){
        $type = $row['IDTIPOCAMPORDSTATION'];
        switch ($type) {
            case 'NUMBER_INPUT':
                $customField['data_type'] = 'INTEGER';
                break;
            case 'CHECK_BOX':
                $customField['data_type'] = 'BOOLEAN';
                break;
            case 'MULTIPLE_CHOICE':
                $customField['data_type'] = 'STRING[]';
                break;
            default:
                $customField['data_type'] = 'STRING';
                break;
        }
        $customField['presentation_type'] = $type;
        if($type === 'CHECK_BOX' || $type === 'MULTIPLE_CHOICE' || $type === 'COMBO_BOX' || $type === 'RADIO_BUTTON'){
            $customField['validation_rules']['valid_options'] = array();
            $options = $row['DSOPCOESCAMPOSELECT'];
            
            foreach($options as $option){
                $formatedOption = array(
                    "value" => $option,
                    "label" => array(
                        "pt-BR" => $option
                    )
                );
                array_push($customField['validation_rules']['valid_options'], $formatedOption);
            }
        }
    }
    
    private function refreshToken($callback, $parameter){
        try{
            $refreshToken = $this->session->get("rdstation_refreshToken");
            if(!$refreshToken){
                $refreshToken = $this->integracaoFactory->getRefreshToken();
            }
            if(!$refreshToken){
                throw new \Exception(self::MISSING_AUTH, 401);
            }
            
            $token = $this->getToken($refreshToken, self::REFRESH);
            if($token){
                if($parameter) {
                    call_user_func_array(array($this, $callback), array($parameter));
                }else {
                    call_user_func(array($this, $callback));
                }
            }    
        }catch(\Exception $e){
            throw new \Exception($e->getMessage(), $e->getCode());    
        }
    }
    
    public function sendContactToRD($row){
        try{
                $contact = array(
                    "token_rdstation"=> $this->integracaoFactory->getPublicTokenRD(),
                    "identificador"=> $row['CONVERSION_EVENT'], 
                    "email"=> $row['EMAIL'],
                    "nome"=> $row['NAME'], 
                    "cargo"=> $row['JOB'],
                    "empresa"=> $row['COMPANY'],
                    "numero-de-funcionarios"=>$row['NR_EMP'], 
                    "endereco-empresa"=>$row['COMPANY_ADDRESS'], 
                    "telefone"=> $row['PHONE'],
                    "celular"=> $row['PHONE_2'],
                    "c_utmz"=>"43135652.1441366582.1.1.utmcsr=Teknisa CRM|utmccn=Teknisa CRM|utmcmd=referral", 
                    "tags"=> $row['TAGS']
                );
                $this->buildCustomFields($contact, $row['NRPARCNEGOCIO']);
                $contact = $this->executeCurl(null, "https://www.rdstation.com.br/api/1.3/conversions", $contact);
                
                return $contact;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'sendContactToRD', $row);
            }
    }
    
    private function buildCustomFields(&$contact, $nrparcnegocio){
        //Pega todos os campos personalizados
        $customFields = $this->integracaoFactory->getCustomFields();
        if(!empty($customFields)){
            $fieldsToSearch = array();
            //Monta um array com os campos e tabelas relacionados aos campos personalizados
            foreach($customFields as $cf){
                $fieldToSearch = array(
                    'FIELD' => $cf['NMCAMPO'],
                    'TABLE' => $cf['NMTABELA']
                );
                if(!empty($cf['NMCAMPORELACIONADO']) && !empty($cf['NMTABELARELACIONADA'])){
                    $fieldToSearch['RELATEDFIELD'] = $cf['NMCAMPORELACIONADO'];
                    $fieldToSearch['RELATEDTABLE'] = $cf['NMTABELARELACIONADA'];
                }
                $fieldsToSearch[] = $fieldToSearch;
            }
            //Monta um array com os valores dos campos personalizados do parceiro
            $fieldsValue = array();
            foreach($fieldsToSearch as $fieldToSearch){
                $field  = $fieldToSearch['FIELD'];
                $table  = $fieldToSearch['TABLE'];
                $select = $field;
                $from   = " $table";
                //@TODO NRORG
                
                //Se não houver campos relacionados, basta buscar na tabela
                if(empty($fieldToSearch['RELATEDFIELD'])){
                    $where  = $table.".NRPARCNEGOCIO = $nrparcnegocio";
                }else{
                //Se houver campos relacionados, monta os joins com as tabelas relacionadas
                    $relatedFields = explode(',', $fieldToSearch['RELATEDFIELD']);
                    $relatedTables = explode(',', $fieldToSearch['RELATEDTABLE']);
                    //Um join para cada campo/tabela relacionada
                    for($i = 0; $i<count($relatedFields); $i++){
                        $relatedTable = $relatedTables[$i];
                        $relatedField = $relatedFields[$i];
                        
                        $from .= " JOIN $relatedTable ON $table.$relatedField = $relatedTable.$relatedField ";
                        $table = $relatedTable;
                    }
                    $where  = $table.".NRPARCNEGOCIO = $nrparcnegocio";
                }
                //Recebe o valor do campo
                $fieldValue = $this->integracaoFactory->getValueOfCustomFields($select, $from, $where);
                $fieldsValue[$field] = $fieldValue[$field];
            }
            //Cria uma posição no array do contato para cada campo personalizado
            $index = 0;
            foreach($fieldsValue as $key => $fieldValue){
                $contact[$customFields[$index]['NMCAMPORDSTATION']] = $fieldsValue[$key];
                $index++;
            }
        }
    }
    
    public function changeFunnel($row){
        try{
                $token = $this->session->get("rdstation_token");
                if(!$token){
                    $e = '{error_type:'.self::MISSING_AUTH.'}';
                    throw new \Exception($e, 401);
                }
                /*$funnel = array(
                    "auth_token"=> $this->integracaoFactory->getPrivateTokenRD(),
                    "lead" => array(
                        "lifecycle_stage" => $row['FUNNEL'],
                        "opportunity" => $this->integracaoFactory->checkIfHasOpenOpport($row['EMAIL'])
                    )
                );
                $funnel = $this->executeCurl(null, 'https://www.rdstation.com.br/api/1.2/leads/'.$row['EMAIL'], $funnel, 'PUT');
                */
                $funnel = array(
                    "lifecycle_stage" => $row['FUNNEL'],
                    isset($row['OPPORTUNITY']) ? $row['OPPORTUNITY'] : (($row['FUNNEL'] === 'Qualified Lead') ? true : false),
                    "contact_owner_email"=> $row['EMAIL']
                );
                $funnel = $this->executeCurl($token, "/platform/contacts/email:" . $row['EMAIL'] . "/funels/default", $funnel, 'PUT');
                return $funnel;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'changeFunnel', $row);
            }
    }
    
    public function markSale($sale){
        try{
            $token = $this->session->get("rdstation_token");
            if($sale['status'] === 'won') {
                $sale = array(
                    "event_type" => "SALE",
                    "event_family" => "CDP",
                    "funnel_name" => "Client",
                    "email" => $sale['email'],
                    "value" => $sale['value']
                );
                $sale = $this->executeCurl($token, '/platform/events', $sale);
            }else {
                $sale = array(
                    "event_type" => "OPPORTUNITY_LOST",
                    "event_family" => "CDP",
                    "funnel_name" => "Qualified Lead",
                    "email" => $sale['email'],
                    "reason" => $sale['lost_reason']
                );
                $sale = $this->executeCurl($token, '/platform/events', $sale);    
            }
            return $sale;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'markSale', $sale);
            }
    }
    
    private function setCustomFields($nrparcnegocio, $customFields){
        if(!empty($customFields)){
            foreach($customFields as $field => $value){
                if(!empty($value)){
                    $filter = "AND NMCAMPORDSTATION = '$field'";
                    $cf = $this->integracaoFactory->getCustomFields($filter, $this->entityManager);
                    $cf = $cf[0];
                    
                    if(empty($cf['NMTABELARELACIONADA'])){
                        $table = $cf['NMTABELA'];
                        $fieldLead = $cf['NMCAMPO'];
                        $this->integracaoFactory->setCustomField($table, $fieldLead, $value, $nrparcnegocio, $this->entityManager);
                    }else{
                        $table = explode(',',$cf['NMTABELARELACIONADA']);
                        $table = $table[0];
                        
                        $select = explode(',',$cf['NMCAMPORELACIONADO']);
                        $select = $select[0];
                        
                        $from  = $cf['NMTABELA'];
                        $nmcampo = $cf['NMCAMPO'];
                        $where = " $nmcampo = '$value' ";
                        
                        $realValue = $this->integracaoFactory->getValueOfCustomFields($select, $from, $where, $this->entityManager);
                        $realValue = $realValue[$select];
                        
                        $this->integracaoFactory->setCustomField($table, $select, $realValue, $nrparcnegocio, $this->entityManager);
                    }
                }
            }    
        }    
    }
    
    private function exceptionHandler($e, $callback, $payload){
        $errorMesage = json_decode($e->getMessage(), true);
        $errorType = (isset($errorMesage[0]) && isset($errorMesage[0]['error_type'])) ? $errorMesage[0]['error_type'] : 'ERROR';
        
        if($errorType == self::UNAUTHORIZED || $e->getCode() == 401){
            try{
                $this->refreshToken($callback, $payload);    
            }catch(\Exception $e){
                throw new \Exception($e->getMessage(), $e->getCode());
            }
        }else{
            if(strpos($e->getMessage(), 'TAKEN') !== false)
                $message = 'Este nome já está vinculado a outro campo.';
            if(strpos($e->getMessage(), 'INVALID') !== false && strpos($e->getMessage(), 'validation_rules') !== false)
                $message = 'As opções não podem conter itens repetidos.';
            if(strpos($e->getMessage(), 'TOO_SHORT') !== false)
                $message = 'As opções devem ser preenchidas.';
            if(strpos($e->getMessage(), 'RESOURCE_NOT_FOUND') !== false)
                $message = 'Campo não encontrado.';
            
            if(!isset($message)){
                $message = $e->getMessage();
                $type    = 1; 
            }else{
                $type    = 2;
            }
            throw new \Exception($message, $type);
        }    
    }
    
    public function feedBackConversion($row){
        try{
                $token = $this->session->get("rdstation_token");
                $eventName = $row['EVENT_NAME'];
                $email = $row['EMAIL'];
                /*Se estiver vindo de uma chamada externa, usa o entityManager externo*/
                if(property_exists($this, 'entityManager'))
                    $entity = $this->entityManager;
                else
                    $entity = null;
                /*OLD     
                $contact = array(
                    "token_rdstation"=> $this->integracaoFactory->getPublicTokenRD($entity),
                    "identificador"=> $eventName, 
                    "email"=> $email,
                    "c_utmz"=>"43135652.1441366582.1.1.utmcsr=Teknisa CRM|utmccn=Teknisa CRM|utmcmd=referral"
                );
                $contact = $this->executeCurl(null, "https://www.rdstation.com.br/api/1.3/conversions", $contact);
                */
                $contact = array(
                    "event_type" => "CONVERSION",
                    "event_family" => "CDP",
                    "payload" => array(
                        "conversion_identifier" => $eventName, 
                        "email"=> $email,
                        "c_utmz"=>"43135652.1441366582.1.1.utmcsr=Teknisa CRM|utmccn=Teknisa CRM|utmcmd=referral"
                    )
                );
                $contact = $this->executeCurl($token, "/platform/events", $contact);
                
                return $contact;
            }catch(\Exception $e){
                $this->exceptionHandler($e, 'feedBackConversion', array($eventName, $email));
            }
    }
    
    public function markOpport($email){
        try{
            $token = $this->session->get("rdstation_token");
            /* OLD
            $opport = array(
                "auth_token"=> $this->integracaoFactory->getPrivateTokenRD(),
                "lead" => array(
                    "opportunity" => true
                )
            );
            $opport = $this->executeCurl(null, 'https://www.rdstation.com.br/api/1.2/leads/'.$email, $opport, 'PUT');
            */
            $opport = array(
                "event_type" => "OPPORTUNITY",
                "event_family" => "CDP",
                "funnel_name" => "Qualified Lead",
                "email" => $email
            );
            $opport = $this->executeCurl($token, '/platform/events', $opport);
            return $opport;
        }catch(\Exception $e){
            $this->exceptionHandler($e, 'markOpport', $email);
        }
    }
    
}
?>