function AbaInterlocutorController(CRM_Utilities, templateManager, ScreenService, Messages, General, metaDataFactory, restEngine) {
    
    /**
     * Insere a widget de interlocutor na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a widget de interlocutor
     * ("WDGT_INTERLOCUTOR").
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setInterlocutorWidgetOnActiveWidget = function(widget, position){
        
        widget.widgets = CRM_Utilities.setElementOnArrayDefinedPosition(this.factoryInterlocutorWidget(widget), widget.widgets, position);
        widget.activate();
        
    };
    
    
    /**
     * builda a widget de interlocutor.
     * 
     * @param {object} widget widget ativa.
     * 
     * @return {object} widget de interlocutor construída.
    */
    this.factoryInterlocutorWidget = function(activeWidget){
        
        var interlocutorWidget = {
    "id": "1722319758311315390519",
    "name": "WDGT_INTERLOCUTOR",
    "label": "Interlocutor",
    "template": "widget/master_detail/grid.html",
    "container": activeWidget.container, 
    "parent": activeWidget,
    "isVisible": true,
    "dataSource": {
        "name": "product#/crud/crm_interlocutor",
        "rest": true,
        "lazyLoad": true
    },
    "dataSourceFilter": [
        {
            "name": "NRPARCNEGOCIO",
            "operator": "=",
            "value": ""
        }
    ],
    "paramsNewCode": {
        "__is_new_code": true,
        "__new_code_pk": "CRM_INTERLOCUTOR",
        "__table_pk": "NRSEQINTERLOCUTOR",
        "__columns_to_return": [
            "NRSEQINTERLOCUTOR"
        ],
        "__new_code_length": 1
    },
    "events": [
        {
            "id": "4704026363171137102435",
            "name": "WidgetOnEnterOnce",
            "code": "AbaInterlocutorController.setAddressFieldsFilterByDefaultOrgAndOperatorOrg(args.owner);"
        },
        {
            "id": "4704026361772684518436",
            "name": "WidgetOnEnter",
            "code": "AbaInterlocutorController.setInterlocutorWidgetFilter(args.owner);PartnerInfosModalController.resizeWindow(); args.owner.reload();"
        },
        {
            "id": "4704026362516730175437",
            "name": "WidgetNewRow",
            "code": "AbaInterlocutorController.setInterlocutorWidgetDefaultValues(args.owner);"
        },
        {
            "id": "12932085961377142479320",
            "name": "WidgetAfterMoveRow",
            "code": "crm00040_Lead_Controller.validateMask(args.owner); crm00040_Lead_Controller.validateMaskPhone(args.owner);"
        }
    ],
    "actions": [
        {
            "id": "12932085961763543151750",
            "name": "ACT_ADICIONAR",
            "isVisible": true,
            "icon": "plus",
            "label": "Adicionar",
            "showAsAction": "always|list",
            "events": [
                {
                    "id": "12932085963772391836751",
                    "name": "ActionEvent",
                    "code": "AbaInterlocutorController.novoInterlocutor(args.owner.widget);"
                }
            ]
        },
        {
            "id": "12932085961137644085752",
            "name": "ACT_EDITAR",
            "isVisible": true,
            "icon": "pencil",
            "label": "Editar",
            "showAsAction": "view",
            "events": [
                {
                    "id": "12932085961785161109753",
                    "name": "ActionEvent",
                    "code": "AbaInterlocutorController.editInterlocutor(args.owner.widget);"
                }
            ]
        },
        {
            "id": "12932085962912644809754",
            "name": "ACT_SALVAR",
            "isVisible": true,
            "label": "Salvar",
            "showAsAction": "never_edit",
            "events": [
                {
                    "id": "12932085961363353803755",
                    "name": "ActionEvent",
                    "code": "AbaInterlocutorController.saveInterlocutor(args.owner.widget);"
                }
            ]
        },
        {
            "id": "12932085962868422568756",
            "name": "ACT_VOLTAR_LISTA",
            "isVisible": true,
            "label": "Voltar",
            "showAsAction": "back_view",
            "events": [
                {
                    "id": "12932085964000722743757",
                    "name": "ActionEvent",
                    "code": "General.widgetGoBackToList(args.owner.widget);"
                }
            ]
        },
        {
            "id": "4704026363618852287433",
            "name": "ACT_VOLTAR_LISTA_PAI",
            "isVisible": true,
            "label": "Voltar",
            "showAsAction": "back_list",
            "events": [
                {
                    "id": "4704026363974312354434",
                    "name": "ActionEvent",
                    "code": "General.widgetGoBackToParent(args.owner.widget);"
                }
            ]
        },
        {
            "id": "129320859675487224758",
            "name": "ACT_CANCELAR_FORMULARIO",
            "isVisible": true,
            "label": "Cancelar",
            "showAsAction": "back_edit",
            "events": [
                {
                    "id": "12932085964021042762759",
                    "name": "ActionEvent",
                    "code": "General.widgetCancelForm(args.owner.widget);"
                }
            ]
        },
        {
            "id": "12932085962468638339760",
            "name": "ACT_DELETAR",
            "isVisible": true,
            "label": "Excluir",
            "icon": "trash",
            "showAsAction": "view",
            "events": [
                {
                    "id": "12932085962865850781761",
                    "name": "ActionEvent",
                    "code": "AbaInterlocutorController.deleteInterlocutor(args.owner.widget);"
                }
            ]
        }
    ],
    "fieldGroups": [
        {
            "id": "4704026362707790522438",
            "name": "CONTATO_INTERLOCUTOR",
            "label": "Contatos",
            "collapsible": true,
            "opened": true
        },
        {
            "id": "470402636407032826439",
            "name": "ENDERECO_INTERLOCUTOR",
            "label": "Endereço",
            "collapsible": true,
            "opened": true
        }
    ],
    "fields": [
        {
            "id": "4704026363644901721399",
            "name": "NRSEQINTERLOCUTOR",
            "label": "",
            "template": "field/text-edit.html",
            "class": "12",
            "isVisible": false,
            "showOnList": false,
            "showOnView": false,
            "showOnForm": false,
            "validations": {}
        },
        {
            "id": "4704026361560207406400",
            "name": "NMINTERLOCUTOR",
            "label": "Nome",
            "template": "field/text-edit.html",
            "class": "12",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "validations": {
                "required": true
            },
            "filterProperties": {
                "validations": {}
            }
        },
        {
            "id": "4704026361402559843401",
            "name": "NRCPFINTERLOC",
            "label": "CPF",
            "template": "field/text-edit.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "maxlength": 14,
            "class": "12",
            "validations": {},
            "events": [
                {
                    "id": "12932085961135018746701",
                    "name": "FieldOnChange",
                    "code": "CRM_Utilities.validaCPF(args.owner.field.widget, 'NRCPFINTERLOC')"
                }
            ]
        },
        {
            "id": "470402636901915548402",
            "name": "DTNASCIMENTO",
            "label": "Data de Nascimento",
            "template": "field/datetime.html",
            "class": "6",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "validations": {},
            "descriptionOrder": 1,
            "datetime-preset": "date",
            "datetime-dateFormat": "dd/mm/yy"
        },
        {
            "id": "4704026363311671195403",
            "name": "IDRECEBEPRESENTE",
            "label": "Recebe Presente",
            "template": "field/select-native.html",
            "class": "6",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "validations": {},
            "valueField": "value",
            "descriptionField": "name",
            "hideEmptyValue": true,
            "skipSearch": true,
            "defaultValue": "S",
            "dataSource": {
                "data": [
                    {
                        "name": "Sim",
                        "value": "S"
                    },
                    {
                        "name": "Não",
                        "value": "N"
                    }
                ]
            },
            "events": []
        },
        {
            "id": "470402636424517917404",
            "name": "NMSETOR",
            "label": "Setor",
            "template": "field/select.html",
            "class": "6",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "validations": {},
            "showSelectAs": "grid",
            "valueField": "CDSETOR",
            "descriptionField": "NMSETOR",
            "dataSource": {
                "name": "product#/crud/setor",
                "rest": true,
                "lazyLoad": true
            },
            "outData": {
                "CDSETOR": "CDSETOR",
                "NMSETOR": "NMSETOR"
            },
            "fields": [
                {
                    "id": "4704026363146147265405",
                    "name": "NMSETOR",
                    "label": "Setor",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ]
        },
        {
            "id": "4704026364283424767406",
            "name": "CDSETOR",
            "label": "",
            "template": "field/text-edit.html",
            "isVisible": false,
            "showOnList": false,
            "showOnForm": false,
            "showOnView": false
        },
        {
            "id": "4704026361324098498407",
            "name": "NMCARGOINTERLOC",
            "label": "Cargo",
            "template": "field/select.html",
            "class": "6",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "validations": {},
            "showSelectAs": "grid",
            "valueField": "CDCARGOINTERLOC",
            "descriptionField": "NMCARGOINTERLOC",
            "dataSource": {
                "name": "product#/crud/cargointerloc",
                "rest": true,
                "lazyLoad": true
            },
            "outData": {
                "CDCARGOINTERLOC": "CDCARGOINTERLOC",
                "NMCARGOINTERLOC": "NMCARGOINTERLOC"
            },
            "fields": [
                {
                    "id": "4704026362335482132408",
                    "name": "NMCARGOINTERLOC",
                    "label": "Cargo",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ],
            "events": []
        },
        {
            "id": "12932085961768620586743",
            "name": "EMAILINTERLOC",
            "label": "Email",
            "template": "field/text-edit.html",
            "class": "12",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "validations": {
                "mail": true
            }
        },
        {
            "id": "12932085962868349647744",
            "name": "NRTELEFONE1",
            "label": "Telefone 1",
            "template": "field/text.html",
            "maxlength": 18,
            "class": "4",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "validations": {}
        },
        {
            "id": "4704026362367364219409",
            "name": "NRRAMALTEL1",
            "label": "Ramal 1",
            "template": "field/text-edit.html",
            "class": "2",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "showOnForm": true,
            "maxlength": 10,
            "validations": {}
        },
        {
            "id": "12932085962868349647744",
            "name": "NRTELEFONE2",
            "label": "Telefone 2",
            "template": "field/text.html",
            "class": "4",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "validations": {}
        },
        {
            "id": "4704026362950630855410",
            "name": "NRRAMALTEL2",
            "label": "Ramal 2",
            "template": "field/text-edit.html",
            "class": "2",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "maxlength": 10,
            "validations": {}
        },
        {
            "id": "12932085962868349647744",
            "name": "NRTELEFONE3",
            "label": "Telefone 3",
            "template": "field/text.html",
            "class": "4",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "validations": {}
        },
        {
            "id": "4704026363408825727411",
            "name": "NRRAMALTEL3",
            "label": "Ramal 3",
            "template": "field/text-edit.html",
            "class": "2",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "maxlength": 10,
            "validations": {}
        },
        {
            "id": "12932085962868349647744",
            "name": "NRTELEFONE4",
            "label": "Telefone 4",
            "template": "field/text.html",
            "class": "4",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "validations": {}
        },
        {
            "id": "4704026363781538409412",
            "name": "NRRAMALTEL4",
            "label": "Ramal 4",
            "template": "field/text-edit.html",
            "class": "2",
            "fieldGroup": "CONTATO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "maxlength": 10,
            "validations": {}
        },
        {
            "id": "4704026361059330171413",
            "name": "NRCEPINTERLOCUTOR",
            "label": "CEP",
            "class": "6",
            "template": "field/text-edit.html",
            "isVisible": true,
            "showOnList": false,
            "showOnForm": true,
            "showOnView": true,
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "validations": {},
            "events": [
                {
                    "id": "4704026363737460426414",
                    "name": "FieldOnChange",
                    "code": "AbaInterlocutorController.fillAddressByCep(args.owner.field.widget);"
                }
            ],
            "maxlength": 15,
            "mask": {
                "type": "fix",
                "params": {
                    "cleanMaskOnModel": true,
                    "mask": "99.999-999"
                }
            },
            "filterProperties": {
                "showOnList": false,
                "showOnForm": false,
                "showOnView": false
            }
        },
        {
            "id": "470402636594185349430",
            "name": "DSENDEINTERLOCUTOR",
            "label": "Endereço",
            "template": "field/text-edit.html",
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnForm": true,
            "showOnView": true,
            "class": "6",
            "validations": {
                "maxLength": {
                    "length": 50
                }
            },
            "events": [],
            "filterProperties": {
                "isVisible": false,
                "showOnList": false,
                "showOnForm": false,
                "showOnView": false
            }
        },
        {
            "id": "4704026363663145192431",
            "name": "NRENDEINTERLOCUTOR",
            "label": "N°",
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "template": "field/number.html",
            "isVisible": true,
            "showOnList": false,
            "showOnForm": true,
            "showOnView": true,
            "class": "6",
            "validations": {},
            "events": [],
            "filterProperties": {
                "showOnList": false,
                "showOnForm": false,
                "showOnView": false
            }
        },
        {
            "id": "4704026362739314762432",
            "name": "DSCOMPINTERLOCUTOR",
            "label": "Complemento",
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "template": "field/text-edit.html",
            "isVisible": true,
            "showOnList": false,
            "showOnForm": true,
            "showOnView": true,
            "class": "6",
            "validations": {},
            "events": [],
            "filterProperties": {
                "showOnList": false,
                "showOnForm": false,
                "showOnView": false
            }
        },
        {
            "id": "4704026361724224837415",
            "name": "NMPAIS",
            "label": "País",
            "template": "field/select.html",
            "class": "6",
            "showSelectAs": "grid",
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "breakline": true,
            "validations": {},
            "dataSource": {
                "name": "product#/crud/pais",
                "rest": true,
                "lazyLoad": true
            },
            "dataSourceFilter": [
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": ""
                }
            ],
            "valueField": "CDPAIS",
            "descriptionField": "NMPAIS",
            "outData": {
                "CDPAIS": "CDPAIS",
                "NMPAIS": "NMPAIS"
            },
            "fields": [
                {
                    "id": "4704026363588634898416",
                    "name": "NMPAIS",
                    "label": "País",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ],
            "events": [
                {
                    "id": "4704026361517789957417",
                    "name": "FieldOnChange",
                    "code": "AbaInterlocutorController.checkCountryChange(args.owner.field.widget.currentRow); AbaInterlocutorController.alteraEstadoReadonly(args.owner.field.widget);"
                },
                {
                    "id": "4704026361390236077418",
                    "name": "FieldOnClick",
                    "code": "args.owner.field.reload();"
                }
            ]
        },
        {
            "id": "4704026361644634769419",
            "name": "NMESTADO",
            "label": "Estado",
            "template": "field/select.html",
            "class": "6",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "readOnly": true,
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "validations": {},
            "showSelectAs": "grid",
            "showAddButtonAs": "button",
            "addButtonText": "Adicionar Estado",
            "dataSource": {
                "name": "product#/crud/estado",
                "rest": true,
                "lazyLoad": true
            },
            "dataSourceFilter": [
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": ""
                },
                {
                    "name": "CDPAIS",
                    "operator": "=",
                    "value": ""
                }
            ],
            "valueField": "SGESTADO",
            "descriptionField": "NMESTADO",
            "outData": {
                "SGESTADO": "SGESTADO",
                "NMESTADO": "NMESTADO"
            },
            "fields": [
                {
                    "id": "4704026361211516554420",
                    "name": "NMESTADO",
                    "label": "Estado",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ],
            "events": [
                {
                    "id": "4704026362138943311421",
                    "name": "FieldOnChange",
                    "code": "AbaInterlocutorController.checkStateChange(args.owner.field.widget.currentRow);AbaInterlocutorController.alteraMunicipioReadonly(args.owner.field.widget);"
                },
                {
                    "id": "1293208596679033120340",
                    "name": "FieldOnAdd",
                    "code": "crm00040_Lead_Controller.criarNovoEstado(args.owner.field.widget)"
                },
                {
                    "id": "4704026362239848735422",
                    "name": "FieldOnClick",
                    "code": "AbaInterlocutorController.placeFilter(args.owner.field.widget, args.owner.field);"
                }
            ]
        },
        {
            "id": "470402636757526035423",
            "name": "NMMUNICIPIO",
            "label": "Município",
            "template": "field/select.html",
            "class": "6",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "readOnly": true,
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "showAddButtonAs": "button",
            "addButtonText": "Adicionar Município",
            "validations": {},
            "dataSource": {
                "name": "product#/crud/municipio",
                "rest": true,
                "lazyLoad": true
            },
            "dataSourceFilter": [
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": ""
                },
                {
                    "name": "SGESTADO",
                    "operator": "=",
                    "value": ""
                }
            ],
            "showSelectAs": "grid",
            "valueField": "CDMUNICIPIO",
            "descriptionField": "NMMUNICIPIO",
            "outData": {
                "NMMUNICIPIO": "NMMUNICIPIO",
                "CDMUNICIPIO": "CDMUNICIPIO"
            },
            "fields": [
                {
                    "id": "470402636485243290424",
                    "name": "NMMUNICIPIO",
                    "label": "Município",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ],
            "events": [
                {
                    "id": "470402636581058328425",
                    "name": "FieldOnChange",
                    "code": "AbaInterlocutorController.checkCityChange(args.owner.field.widget.currentRow);AbaInterlocutorController.alteraBairroReadonly(args.owner.field.widget);"
                },
                {
                    "id": "12932085962049539827220",
                    "name": "FieldOnAdd",
                    "code": "crm00040_Lead_Controller.criarNovoMunicipio(args.owner.field.widget)"
                },
                {
                    "id": "4704026363008653175426",
                    "name": "FieldOnClick",
                    "code": "AbaInterlocutorController.placeFilter(args.owner.field.widget, args.owner.field);"
                }
            ]
        },
        {
            "id": "4704026361521773731427",
            "name": "NMBAIRRO",
            "label": "Bairro",
            "template": "field/select.html",
            "class": "6",
            "isVisible": true,
            "showOnList": false,
            "showOnView": true,
            "showOnForm": true,
            "readOnly": true,
            "maxlength": 100,
            "showAddButtonAs": "button",
            "addButtonText": "Adicionar Bairro",
            "showSelectAs": "grid",
            "fieldGroup": "ENDERECO_INTERLOCUTOR",
            "validations": {},
            "dataSource": {
                "name": "product#/crud/bairro",
                "rest": true,
                "lazyLoad": true
            },
            "dataSourceFilter": [
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": ""
                },
                {
                    "name": "CDMUNICIPIO",
                    "operator": "=",
                    "value": ""
                }
            ],
            "valueField": "CDBAIRRO",
            "descriptionField": "NMBAIRRO",
            "outData": {
                "CDBAIRRO": "CDBAIRRO",
                "NMBAIRRO": "NMBAIRRO"
            },
            "fields": [
                {
                    "id": "4704026363686845137428",
                    "name": "NMBAIRRO",
                    "label": "Bairro",
                    "isVisible": true,
                    "showOnList": true,
                    "template": "field/text-edit.html"
                }
            ],
            "events": [
                {
                    "id": "12932085962726970312221",
                    "name": "FieldOnAdd",
                    "code": "crm00040_Lead_Controller.criarNovoBairro(args.owner.field.widget);"
                },
                {
                    "id": "4704026361487655038429",
                    "name": "FieldOnClick",
                    "code": "AbaInterlocutorController.placeFilter(args.owner.field.widget, args.owner.field);"
                }
            ]
        }
    ],
    "widgets": [
        {
            "id": "12932085962659776778",
            "name": "popUpNovoSetor",
            "label": "Novo Setor",
            "container": activeWidget.container,
            "isVisible": true,
            "showAsTab": false,
            "template": "widget/form.html",
            "events": [],
            "dataSource": {
                "name": "product#/crud/setor",
                "rest": true,
                "lazyLoad": true
            },
            "paramsNewCode": {
                "__is_new_code": true,
                "__new_code_pk": "SETOR",
                "__table_pk": "CDSETOR",
                "__new_code_column_size": 4,
                "__columns_to_return": [
                    "CDSETOR"
                ]
            },
            "actions": [
                {
                    "id": "129320859630533787387",
                    "name": "BACK_EDIT",
                    "label": "Cancelar",
                    "isVisible": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "id": "12932085962959932781",
                            "name": "ActionEvent",
                            "code": "crm00040_Lead_Controller.cancelarNovoRegistro()"
                        }
                    ]
                },
                {
                    "id": "12932085962627022",
                    "name": "SAVE",
                    "label": "Salvar",
                    "isVisible": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "id": "129321863783",
                            "name": "ActionEvent",
                            "code": "AbaInterlocutorController.saveSetor(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "12932085962763145692779",
                    "name": "SGSETOR",
                    "label": "Sigla do Setor",
                    "class": "4",
                    "maxlength": 10,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "readOnly": false,
                    "validations": {
                        "required": true
                    },
                    "template": "field/text-edit.html"
                },
                {
                    "id": "12932085962763145692779",
                    "name": "NMSETOR",
                    "label": "Setor",
                    "class": "8",
                    "maxlength": 30,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "readOnly": false,
                    "validations": {
                        "required": true
                    },
                    "template": "field/text-edit.html"
                }
            ]
        }
    ]
};
        
        interlocutorWidget = metaDataFactory.widgetFactory(interlocutorWidget);
        CRM_Utilities.factoryWidgetActions(interlocutorWidget);
        
        return interlocutorWidget;
        
    };
    
    this.openPopupNovoSetor = function(widget){
        var widSetor = templateManager.container.getWidget('popUpNovoSetor');
        ScreenService.openPopup(widSetor).then(function(){
            widSetor.edit();
            widSetor.template = "widget/form.html";
            widSetor.activate();
            widSetor.currentRow = {};  
            widSetor.actions = widSetor.actions.splice(0,2);
        });
    };
    
    this.saveCargo = function(widget){
        //var widInter = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        CRM_Utilities.setColumnToReturnInCurrentRow(widget.currentRow, 'CDCARGOINTERLOC');
        
        widget.currentRow.NRORG = getLocalVar("CRM_NRORGTRAB");
        widget.currentRow.__new_code_column_size = 4;
        General.widgetSave(widget).then(function(response){
            widget.reload();
        });
        
    };
    
    this.saveSetor = function(widget){
        //var widInter = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        CRM_Utilities.setColumnToReturnInCurrentRow(widget.currentRow, 'CDSETOR');
        
        widget.currentRow.__new_code_column_size = 4;
        widget.currentRow.CDDEPTO = '';
        General.widgetSave(widget).then(function(response){
            widget.reload();
            //widInter.currentRow.CDSETOR = response.dataset.columnsToReturn[0].CDSETOR;
            //widInter.currentRow.NMSETOR = widget.currentRow.NMSETOR;
            //ScreenService.showMessage('Setor adicionado com sucesso!');
        });
        
    };
    
    /**
     * Define o filtro de acordo com o local referente selecionado,
     * por exemplo, filtra apenas os municípios do estado selecionado.
     * 
     * @param {object} widget widget pai da widget de interlocutor
    */
    this.placeFilter = function(widget,field){
        var placeName = field.name;
        var currentRow = widget.currentRow;
        var filter;
        var filterCode;
        
        switch(placeName){
            case "NMESTADO":
                filterCode = "CDPAIS";
                filter = this.createEqualFilter(filterCode,currentRow.CDPAIS);
            break;
            
            case "NMMUNICIPIO":
                filterCode = "SGESTADO";
                filter = this.createEqualFilter(filterCode,currentRow.SGESTADO);
            break;
            
            case "NMBAIRRO":
                filterCode = "CDMUNICIPIO";
                filter = this.createEqualFilter(filterCode,currentRow.CDMUNICIPIO);
            break;
        }
        
        var localNameIndexFilter = CRM_Utilities.getFilterPosition(field.dataSourceFilter, filterCode);
        
        if(field.dataSourceFilter[localNameIndexFilter]){
            field.dataSourceFilter = CRM_Utilities.deleteFilterObject(field.dataSourceFilter, localNameIndexFilter);
        }
        
        field.dataSourceFilter.push(filter);
        field.reload();
    };
    
    /**
     * Retorna um objeto "filtro" com o operador
     * de igualdade.
     * 
     * @param {string} name campo referente para filtrar
     * 
     * @param {string || number} value valor a ser filtrado
     * 
     * @return {object} objeto como filtro.s
    */
    this.createEqualFilter = function(name,value){
        var filter = {
            "name": name,
            "operator": "=",
            "value": value
        };
            
        return filter;
    };
    
    /**
     * Verifica se o usuário modificou o país selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedCountry;
    this.checkCountryChange = function(currentRow){
        var currentCountrys = currentRow.NMPAIS.toString();
        
        if(typeof selectedCountry === "undefined"){
            selectedCountry = currentCountrys;
        }else if(selectedCountry !== currentCountrys){
            currentRow.NMESTADO = "";
            currentRow.SGESTADO = "";
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedCountry = currentCountrys;
        }
    };
    
    /**
     * Verifica se o usuário modificou o estado selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedState;
    this.checkStateChange = function(currentRow){
        var currentState = currentRow.NMESTADO.toString();
        
        if(typeof selectedState === "undefined"){
            selectedState = currentState;
        }else if(selectedState !== currentState){
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedState = currentState;
        }
    };
    
    /**
     * Verifica se o usuário modificou o cidade selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    var selectedCity;
    this.checkCityChange = function(currentRow){
        var currentMunicipality = currentRow.NMMUNICIPIO.toString();
        
        if(typeof selectedCity === "undefined"){
            selectedCity = currentMunicipality;
        }else if(selectedCity !== currentMunicipality){
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedCity = currentMunicipality;
        }
    };
    
    /**
     * Define o bloqueio do campo de estado de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraEstadoReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit" || widget.mode === "list") {
            var paisField = widget.getField("NMPAIS");
            var estadoField = widget.getField("NMESTADO");
            var currentRow = widget.currentRow;
            if (!paisField.value()) {
                estadoField.readOnly = true;
                currentRow.SGESTADO = "";
                currentRow.NMESTADO = "";
            }
            else {
                estadoField.readOnly = false;
            }
            this.alteraMunicipioReadonly(widget);
            
        }
    };
    
    /**
     * Define o bloqueio do campo de município de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraMunicipioReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit" || widget.mode === "list") {
            var estadoField = widget.getField("NMESTADO");
            var municipioField = widget.getField("NMMUNICIPIO");
            var currentRow = widget.currentRow;
            if (!estadoField.value()) {
                municipioField.readOnly = true;
                currentRow.NMMUNICIPIO = "";
                currentRow.CDMUNICIPIO = "";
                currentRow.NMMUNICIPIO = "";
            }
            else {
                municipioField.readOnly = false;
            }
        }
        this.alteraBairroReadonly(widget);
    };
    
    /**
     * Define o bloqueio do campo de bairro de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraBairroReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit" || widget.mode === "list"){
            var municipioField = widget.getField("NMMUNICIPIO");
            var bairroField = widget.getField("NMBAIRRO");
            var currentRow = widget.currentRow;
            if (!municipioField.value()) {
                bairroField.readOnly = true;
                currentRow.NMBAIRRO = "";
                currentRow.CDBAIRRO = "";
                currentRow.NMBAIRRO = "";
            }
            else {
                bairroField.readOnly = false;
            }
        }
    };
    
    /**
     * Define o filtro dos campos de endereço
     * pela organização padrão do sistema e
     * pela organização do operador logado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.setAddressFieldsFilterByDefaultOrgAndOperatorOrg = function(widget){
        
        var addressFields = [widget.getField('NMPAIS'), widget.getField('NMESTADO'), widget.getField('NMMUNICIPIO'), widget.getField('NMBAIRRO')];
        
        addressFields.forEach(function(field){
            this.setAddressFieldFilterValue(field);
        }.bind(this));
        
    };
    
    /**
     * Define o filtro do campo recebido como parâmtro
     * para buscar registros da organização padrão e
     * da organização do operador logado.
     * 
     * @param {object} field campo de endereço a ser
     * filtrado.
    */
    this.setAddressFieldFilterValue = function(field){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        var fieldFilter = field.dataSourceFilter;
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "NRORG");
        
        if(Util.isUndefined(nrorgFilterIndex)){
            nrorgFilterIndex = field.dataSourceFilter.push({name: "NRORG", operator: "IN", value: ""}) - 1;
        }    
        
        fieldFilter[nrorgFilterIndex].value = [loggedOperatorInfos.NRORG, getLocalVar('CRM_NRORGPADRAO')];
        
    };
    
    /**
     * Define os valores iniciais do cadastro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor ('WDGT_INTERLOCUTOR').
    */
    this.setInterlocutorWidgetDefaultValues = function(widget){
        
        var currentRow = widget.currentRow;
        var parentWidget = widget.getParent();
        
        currentRow.NRPARCNEGOCIO = parentWidget.currentRow.NRPARCNEGOCIO;
        
    };
    
    /**
     * Salve um registro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor('WDGT_INTERLOCUTOR').
    */
    this.saveInterlocutor = function(widget) {
        var nrParcNegocio = widget.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
        
        if (!nrParcNegocio) {
            nrParcNegocio = widget.parent.parent.currentRow.NRPARCNEGOCIO;
        }
        
        if (!tipoParc) {
            if (templateManager.container.name == 'ger45000_prospect') {
                tipoParc = "PROSPECT";
            }
            if (templateManager.container.name == 'crm_cliente') {
                tipoParc = "CLIENTE";
            }
        }
        
        if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
            this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
                if (canEdit) {
                    var currentRow = widget.currentRow;
                    var newRow = widget.currentRow.__is_new;
                    
                    this.removeInterlocutorPhoneMask(widget);
                    
                    if (Util.isEmptyOrBlank(currentRow.NRCPFINTERLOC)) {
                        currentRow.NRCPFINTERLOC = '';
                    }
                    
                    currentRow.NRCPFINTERLOC = currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
                    CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRSEQINTERLOCUTOR");
                    
                    General.widgetSave(widget).then(function(response) {
                        currentRow.NRSEQINTERLOCUTOR = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR;
                        if (newRow) { // A função de salvar não recarrega caso o registro seja novo
                            widget.reload();
                        }
                    });
                } else {
                    this.showMessageCantEditParc();
                }
            }.bind(this));
        } else {
            var currentRow = widget.currentRow;
            var newRow = widget.currentRow.__is_new;
            
            this.removeInterlocutorPhoneMask(widget);
            
            if (Util.isEmptyOrBlank(currentRow.NRCPFINTERLOC)) {
                currentRow.NRCPFINTERLOC = '';
            }
            
            currentRow.NRCPFINTERLOC = currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
            CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRSEQINTERLOCUTOR");
            
            General.widgetSave(widget).then(function(response) {
                currentRow.NRSEQINTERLOCUTOR = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR;
                if (newRow) { // A função de salvar não recarrega caso o registro seja novo
                    widget.reload();
                }
            });
        }
    };
    
    /**
     * Retira a máscara dos telefones digitados
     * na currentRow.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.removeInterlocutorPhoneMask = function(widget){
        
        var currentRow = widget.currentRow;
        var phoneFields = ["NRTELEFONE1", "NRTELEFONE2", "NRTELEFONE3", "NRTELEFONE4"];
        
        phoneFields.forEach(function(phone){
            
            if(currentRow[phone]){
                currentRow[phone] = currentRow[phone].replace(/[^0-9]/g, '');    
            }
            
        });
        
    };
    
    /**
     * Abre um ambiente para edição de um
     * interlocutor.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.editInterlocutor = function (widget) {
        var nrParcNegocio = widget.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;

        if (widget.parent.currentRow.NRPARCPROPRI === CRM_Utilities.getLoggedOperatorInfos().NRPARCNEGOCIO || widget.parent.currentRow.NRPARCRESPROSP === CRM_Utilities.getLoggedOperatorInfos().NRPARCNEGOCIO) {

            if (!nrParcNegocio) {
                nrParcNegocio = widget.parent.parent.currentRow.NRPARCNEGOCIO;
            }

            if (!tipoParc) {
                if (templateManager.container.name == 'ger45000_prospect') {
                    tipoParc = "PROSPECT";
                }
                if (templateManager.container.name == 'crm_cliente') {
                    tipoParc = "CLIENTE";
                }
            }

            if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
                this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function (canEdit) {
                    if (canEdit) {
                        General.widgetEdit(widget);

                        this.checkAddressFieldsPermission(widget);
                    } else {
                        this.showMessageCantEditParc();
                    }
                }.bind(this));
            } else {
                General.widgetEdit(widget);

                this.checkAddressFieldsPermission(widget);
            }
        } else {
            var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
            tipoParc == "PROSPECT" || tipoParc == 'CLIENTE' || widget.parent.name === "gridProspect" || widget.parent.name === "gridCliente" ? this.showMessageCantEditParc() : this.showMessageCantEditLead();
        }
    };
    
    /**
     * Define as permissões dos campos de endereço,
     * se eles estarão bloqueados ou não.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.checkAddressFieldsPermission = function(widget){
        
        this.alteraEstadoReadonly(widget);
        this.alteraMunicipioReadonly(widget);
        this.alteraBairroReadonly(widget);    
        
    };
    
    /**
     * Preenche os campos de endereço pelo CEP
     * digitado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.fillAddressByCep = function(widget) {
        
        var currentRow = widget.currentRow;
        var cep = currentRow.NRCEPINTERLOCUTOR;
        
        if(Util.isEmptyOrBlank(cep)) {
            ScreenService.showMessage("CEP não informado!");
        } else if(cep.length === 8) {
            CRM_Utilities.buscarCep(cep).then(function(response) {
                var foundAddress = response.dataset.ENDERECO;
                
                currentRow.NMPAIS = "Brasil";
                currentRow.CDPAIS = "0055";
                currentRow.NMESTADO = foundAddress.NMESTADO;
                currentRow.SGESTADO = foundAddress.SGESTADO;
                currentRow.NMMUNICIPIO = foundAddress.NMMUNICIPIO;
                currentRow.CDMUNICIPIO = foundAddress.CDMUNICIPIO;
                currentRow.NMBAIRRO = foundAddress.NMBAIRRO;
                currentRow.CDBAIRRO = foundAddress.CDBAIRRO;
                currentRow.DSENDEINTERLOCUTOR = foundAddress.DSENDERECO;
                
                this.checkAddressFieldsPermission(widget);
                
                /*TODO Encontrar um jeito melhor de fazer o autocomplete atualizar os fields para os dados da currentRow*/
                angular.element('#footer').click();
            }.bind(this)).catch(function(error) {
                ScreenService.showMessage(Messages.ENDERECO_NAO_ENCONTRADO);
            });    
        } else {
            ScreenService.showMessage(Messages.CEP_INVALIDO);
        }
    };
    
    /**
     * Define o filtro da aba de interlocutor pelo
     * parceiro de negócio em caso.
     * 
     * @param {object} widget widget de interlocutor.
    */
    this.setInterlocutorWidgetFilter = function(widget){
        
        var parentWidgetCurrentRow = widget.getParent().currentRow;
        /*Adaptação para widget modal 'dados do parceiro'*/
        if(Util.isEmptyOrBlank(Object.keys(parentWidgetCurrentRow))){
            parentWidgetCurrentRow = widget.getParent().widgets[0].currentRow;
        }
        var nrparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(widget.dataSourceFilter, "NRPARCNEGOCIO");
        if(!Boolean(widget.NRPARCEIROFILTRADO) || widget.NRPARCEIROFILTRADO !== parentWidgetCurrentRow.NRPARCNEGOCIO){
            
             widget.dataSourceFilter[nrparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
             widget.NRPARCEIROFILTRADO = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
             widget.reload();
            
        }
        
    };
    
    this.novoInterlocutor = function(widget) {
        var nrParcNegocio = widget.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
    
        if (!nrParcNegocio) {
            nrParcNegocio = widget.parent.parent.currentRow.NRPARCNEGOCIO;
        }
    
        if (!tipoParc) {
            if (templateManager.container.name == 'ger45000_prospect') {
                tipoParc = "PROSPECT";
            }
            if (templateManager.container.name == 'crm_cliente') {
                tipoParc = "CLIENTE";
            }
        }
    
        if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
            this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
                if (canEdit) {
                    General.widgetNewRow(widget);
                } else {
                    this.showMessageCantEditParc();
                }
            }.bind(this));
        } else {
            General.widgetNewRow(widget);
        }
    };
    
    this.deleteInterlocutor = function (widget) {
        var nrParcNegocio = widget.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
        if (widget.parent.currentRow.NRPARCPROPRI === CRM_Utilities.getLoggedOperatorInfos().NRPARCNEGOCIO || widget.parent.currentRow.NRPARCRESPROSP === CRM_Utilities.getLoggedOperatorInfos().NRPARCNEGOCIO) {
            if (!nrParcNegocio) {
                nrParcNegocio = widget.parent.parent.currentRow.NRPARCNEGOCIO;
            }

            if (!tipoParc) {
                if (templateManager.container.name == 'ger45000_prospect') {
                    tipoParc = "PROSPECT";
                }
                if (templateManager.container.name == 'crm_cliente') {
                    tipoParc = "CLIENTE";
                }
            }

            if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
                this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function (canEdit) {
                    if (canEdit) {
                        General.deleteCrud(widget);
                    } else {
                        this.showMessageCantEditParc();
                    }
                }.bind(this));
            } else {
                General.deleteCrud(widget);
            }
        } else {
            var tipoParc = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
            tipoParc == "PROSPECT" || tipoParc == 'CLIENTE' || widget.parent.name === "gridProspect" || widget.parent.name === "gridCliente" ? this.showMessageCantEditParc() : this.showMessageCantEditLead();
        }
    };
    
    this.showMessageCantEditLead = function() {
        var message = "Ação bloqueada!<br>" +
            "Somente o operador responsável pelo lead e seus assistentes podem" +
            "editá-lo/evoluí-lo.";
    
        ScreenService.showMessage(message);
    };

    this.showMessageCantEditParc = function() {
        var message = "Ação bloqueada!<br>" +
            "Somente o operador responsável pelo prospect e seus assistentes podem<br>" +
            "editá-lo/evoluí-lo.";
    
        ScreenService.showMessage(message);
    };
    
    this.checkIfCanEditOrEvolParc = function(nrParcNegocio) {
        var restRow = {
            NRPARCNEGOCIO: nrParcNegocio,
            CRM_NRORGTRAB: getLocalVar('CRM_NRORGTRABALHO')
        };
        
        var params = {
            requestType: 'Row',
            row: restRow
        };
        
        return restEngine.post('/checkIfCanEditOrEvolParc', params)
            .then(function(response) {
                return response.dataset.DADOS.CAN_EDIT;
            })
            .catch(function() {
                return false;
            });
    };
    
    this.checkTelephoneMask = function() {
        var widget = templateManager.container.getActiveWidget();
        var nmTipoComu = widget.getField('NMFORMACOMU');
        
        if(nmTipoComu == 'Telefone Internacional') {
            
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('AbaInterlocutorController', AbaInterlocutorController);
});