function crm00070000_Hierarquia_Operador_Controller(CRM_Utilities, templateManager, ScreenService, General, ZHPromise) {
    
    /**
     * Defini o filtro do grid de operadores da hirarquia
     * selecionada.
     * 
     * @param {object} widget widget com a lista dos operadores
     * relacionado a hierarquia selecionada('WDGT_LISTA_OPERADORES_HIERARQUIA').
    */
    this.setHierarchyOperatorsWidgetFiler = function(widget){
        
        var treeGridWidget = widget.parent.parent;
        var treeGridWidgetCurrentRow = treeGridWidget.currentRow;
        
        var parentHierarchyNumberIndex = CRM_Utilities.getObjectPositionInArrayByName(widget.dataSourceFilter, "NRHIERARQUIAPAI");
        
        widget.dataSourceFilter[parentHierarchyNumberIndex].value = treeGridWidgetCurrentRow.NRHIERARQUIAOPER;
    };
    
    /**
     * Salva uma nova hierarquia.
     * 
     * @param {object} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.saveHierarchy = function(widget){
        
        var currentRow = widget.currentRow;
        CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRHIERARQUIAOPER");
        
        if(widget.isValid()){
            
            if(!Boolean(currentRow.NRHIERARQUIAOPERRESP)) {
                
                General.widgetSave(widget).then(function(response){
                    
                    currentRow.NRHIERARQUIAOPERRESP = response.dataset.columnsToReturn[0].NRHIERARQUIAOPER;
                    this.saveHierarchyOperators(widget).then(function() {
                        widget.reload();
                    });
                    
                }.bind(this));
                    
            } else {
                
                this.saveHierarchyOperators(widget).then(function(){
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });   
                });
                
            }
            
            if(Boolean(widget.ISTRANSFER)){
                
                this.returnOperRespFieldDefaultFilter(widget.getField('DSHIERARQUIAOPER'));
                this.deleteTransferParamOnWidget(widget);
                
            }
            
        }
        
    };
    
    /**
     * Salva os operadores de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.saveHierarchyOperators = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(Boolean(currentRow.CDOPERADORESHIERAR) && currentRow.CDOPERADORESHIERAR.length > 0){
            
            var hierarchyOperators = this.prepareHierarchyOperatorDatasToSave(currentRow);
            return General.callRestEngine("product#/crud/crm_hierarquiaoper/save", "DataSet", hierarchyOperators);
            
        } else {
            
            var zhPromise = ZHPromise.defer();
            zhPromise.resolve();
            
            return zhPromise.promise;
        }
        
    };
    
    /**
     * Prepara os registros dos operadores para serem
     * salvos na hierarquia em caso.
     * 
     * @param {object} currentRow currentRow da hierarquia
     * que está sendo criada;
    */
    this.prepareHierarchyOperatorDatasToSave = function(currentRow){
        
        var operators = [];
        var object;
        
        currentRow.CDOPERADORESHIERAR.forEach(function(operator,i){
            
            object = {};
            
            CRM_Utilities.setColumnToReturnParamsOnRow(object, "CRM_HIERARQUIAOPER", "NRHIERARQUIAOPER", "NRHIERARQUIAOPER");
            
            object.__is_new = "__is_new";
            object.CDOPERADOR = currentRow.CDOPERADORESHIERAR[i];
            object.DSHIERARQUIAOPER = currentRow.NMOPERADORHIERAR[i].toUpperCase();    
            object.NRHIERARQUIAPAI = currentRow.NRHIERARQUIAOPERRESP;
            object.IDOPERMASTER = 'N';
            
            operators.push(object);
            
        });
        
        return operators;
        
    };
    
    /**
     * Tira a possibilidade do operador selecionado como
     * responsável ser selecionado como operador relacionado.
     * 
     * @param {object} field field de operadores relacionados.
    */
    this.setRelatedOperatorsFilter = function(field){
        
        var widget = field.widget;
        var currentRow = widget.currentRow;
        
        var fieldFilter = field.dataSourceFilter;
        var cdoperadorFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(fieldFilter, "CDOPERADOR");
        
        fieldFilter[cdoperadorFilterIndex].value = currentRow.CDOPERADOR;
        
    };
    
    /**
     * Executa as funções da widget no evento
     * "WidgetAfterMoveRow".
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.widgetAfterMoveRow = function(widget){
        
        this.setActionToDeleteVisibilityBySelectedHierarchy(widget);
        this.setSuperiorHierarchyFilter(widget);
        
    };
    
    /**
     * Define a visibilidade da ação de excluir hierarquias.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.setActionToDeleteVisibilityBySelectedHierarchy = function(widget){
        
        var currentRow = widget.currentRow;
        
        var actionToDeleteIndex = CRM_Utilities.getObjectPositionInArrayByName(widget.actions, "DELETAR_HIERARQUIA");
        
        if(currentRow._childCount !== 0) {
            widget.actions[actionToDeleteIndex].isVisible = false;
        } else {
            widget.actions[actionToDeleteIndex].isVisible = true;
        }
        
    };
    
    /**
     * Habilita a edição de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.editHierarchy = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(currentRow._childCount !== 0){
            this.transferHierarchy(widget);
        } else {
            General.widgetEdit(widget);
            this.returnOperRespFieldDefaultFilter(widget.getField('DSHIERARQUIAOPER'));
            this.deleteTransferParamOnWidget(widget);
        }
        
    };
    
    /**
     * Habilita a transferência de uma hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.transferHierarchy = function(widget){
        
        General.widgetEdit(widget);
        
        this.setTransferParamsOnWidget(widget);
        this.setFilterRespOperByOperWithoutHierarchy(widget.getField('DSHIERARQUIAOPER'));
        
    };
    
    /**
     * Seta o filtro do campo de operador responsável
     * para buscar apenas operadores sem hierarquia.
     * 
     * @param {object} field campo de operador responsável.
    */
    this.setFilterRespOperByOperWithoutHierarchy = function(field){
        
        var fieldFilter = field.dataSourceFilter;
        
        var filter = {
            name: "NRHIERARQUIAOPER",
            operator: "IS_NULL"
        };
        
        var nrhierarquiaoperFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(fieldFilter, "NRHIERARQUIAOPER");
        
        if(!Boolean(nrhierarquiaoperFilterIndex)){
            
            fieldFilter.push(filter);
            field.reload();    
            
        }
        
    };
    
    /**
     * Retorna o filtro do campo "Operador responsável"
     * para o padrão.
     * 
     * @param {object} field campo de operador responsável.
    */
    this.returnOperRespFieldDefaultFilter = function(field) {
        
        var fieldFilter = field.dataSourceFilter;
        var nrhierarquiaoperFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(fieldFilter, "NRHIERARQUIAOPER");
        
        if(nrhierarquiaoperFilterIndex === 0 || nrhierarquiaoperFilterIndex){
            
            field.dataSourceFilter = CRM_Utilities.deleteFilterObject(fieldFilter, nrhierarquiaoperFilterIndex);
            field.reload();
            
        }
        
    };
    
    /**
     * Define um parâmetro na widget indicando uma
     * trânferencia de hierarquia.
     * 
      * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.setTransferParamsOnWidget = function(widget){
        
        widget.ISTRANSFER = true;    
        
    };
    
    /**
     * Exclui o parâmetro na widget indicando uma
     * trânferencia de hierarquia.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.deleteTransferParamOnWidget = function(widget){
        
        if(Boolean(widget.ISTRANSFER)){
            delete widget.ISTRANSFER;    
        }
        
    };
    
    /**
     * Deleta uma hierarquia e atualiza a widget
     * em seguida.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.deleteHierarchy = function(widget){
        
        General.deleteCrud(widget).then(function(){
           widget.reload(); 
        });
        
    };
    
    /**
     * Define o filtro do campo "Hierarquia Superior" para
     * retornar apenas hierarquias superiores ou no mesmo nível
     * da hierarquia selecionada.
     * 
     * @param {type} widget widget principal da tela
     * de hierarquia ("WDGT_ARVORE_HIERARQUIA").
    */
    this.setSuperiorHierarchyFilter = function(widget){
        
        var currentRow = widget.currentRow;
        
        var SuperiorHierarchyField = widget.getField('DSHIERARQUIAOPERPAI');
        var fieldFilter = SuperiorHierarchyField.dataSourceFilter;
        
        var nrhierarquiaoperFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(fieldFilter, "P_HIERARQUIATUAL");
        
        fieldFilter[nrhierarquiaoperFilterIndex].value = currentRow.NRHIERARQUIAOPER;
        
        SuperiorHierarchyField.reload();
        
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm00070000_Hierarquia_Operador_Controller', crm00070000_Hierarquia_Operador_Controller);
});