function crmLoginController(ScreenService, Operador_Repository, eventAggregator, restEngine, localStorageEngine, Messages, templateManager, API_GoogleCalendar, General, CRM_Utilities, CRM_Operador_Repository) {
    
    /**
     * Evento para iniciar o sistema principal(CRM);
    */
    eventAggregator.subscribe('authSuccess', function(session) {
        
        this.setLoggedOperatorInfosIntemplateManager(session).then(function(){
            this.setAlterarParametros(session);
            this.getInferiorHierarchiesByLoggedOperator(templateManager.LOGGEDOPERATOR).then(function(){
               this.initializeSystem(session); 
            }.bind(this));
            
        }.bind(this));
        
    }.bind(this));
    
    /**
     * Função para iniciar o sistema com 
     * todas as configurações necessárias.
     * 
     * @param {object} session informações da sessão em caso.
    */
    this.initializeSystem = function(session){
        
        API_GoogleCalendar.loadAuth2AndApiClientLibrary();
        
        this.openCrmDashboard().then(function(){
            this.setCrmParametrizationInTemplateManager().then(function(){
                //this.changeAclOperatorWindowToCrmOperatorWindow();
                this.setSystemValidation();
            }.bind(this));    
        }.bind(this));
        
    };
    
    /**
     * Inicializa validações customizadas do sistema.
    */
    this.setSystemValidation = function(){
        
        CRM_Utilities.registerPhoneValidation();
        CRM_Utilities.registerMinCurrencyValidation();
        
    };
    
    /**
     * Função para chamar a tela de dashboard
     * do CRM.
    */
    this.openCrmDashboard = function(){
        return ScreenService.openWindow('product#dashboard');
    };
    
    //TODO
    this.changeAclOperatorWindowToCrmOperatorWindow = function(){
        templateManager.project.getMenu('acl').menus[0].windowName = 'product#crm_operador';
    };
    
    /**
     * Função para setar as informações do operador
     * logado na sessão do PHP.
     * 
     * @param {object} user objeto com as informações
     * do operador logado.
     * 
     * @param {object} resultado da requisição ao
     * back-end.
    */
    this.setLoggedOperatorInfosInPhpSession = function(){
        
        var loggedOperatorInfos = {
            "CDOPERADOR": localStorageEngine.getLocalVar('CRM_CDOPERADOR'),
            "NRORG"     : localStorageEngine.getLocalVar('CRM_NRORG'),
            "NRORGTRAB" : localStorageEngine.getLocalVar('CRM_NRORGTRAB'),
            "CDFILIAL"  : 9999
        };
        
        return restEngine.post("product#/setLoggedOperatorInfosInPhpSession", {
            "requestType": "Row",
            "row": loggedOperatorInfos
        });    
        
    };
    
    /**
     * Função para buscar as informações do operador logado,
     * através de uma requisição no banco, e retorná-las para
     * dentro do templateManager.
     * 
     * @param {object} session sessão da autenticação do usuário
     * logado.
    */
    this.setLoggedOperatorInfosIntemplateManager = function(session){
        var cdoperador = 'OPERADOR_LOGADO';
        if(!Util.isEmptyOrBlank(session) && !Util.isEmptyOrBlank(session.user)) {
            cdoperador = session.user.cdoperador;
        }
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": cdoperador
            }
        ];
        
        return Operador_Repository.retornaOperadorLogado(session.user).then(function(operator){
            if(!operator[0]){
                operator = localStorageEngine.download("product#/crud/operador_geral", filter);
                if(!operator[0]) {
                    operator = localStorageEngine.download("product#/crud/operador_geral", filter);
                    if(!operator[0]) {
                        ScreenService.showMessage(Messages.ERRO_REQUSICAO_INFOS_OPERADOR_LOGADO);
                        return;
                    }
                }
            }
        
            templateManager.LOGGEDOPERATOR = operator[0];
            Operador_Repository.retornaPerfilOperadorLogado().then(function(perfil){
                templateManager.LOGGEDOPERATOR.NRSEQPERFATIV = null;
                if(Array.isArray(perfil)){
                    templateManager.LOGGEDOPERATOR.NRSEQPERFATIV = [];
                    templateManager.LOGGEDOPERATOR.DESCPEFATIV = [];
                    perfil.forEach(function(perf){
                        if(templateManager.LOGGEDOPERATOR.NRSEQPERFATIV.indexOf(perf.NRSEQPERFATIV) === -1){
                            templateManager.LOGGEDOPERATOR.NRSEQPERFATIV.push(perf.NRSEQPERFATIV);
                            templateManager.LOGGEDOPERATOR.DESCPEFATIV.push(perf.DESCPEFATIV);
                        }
                    });
                }
            });

            }.bind(this), 
            function(error){
                ScreenService.showMessage(Messages.ERRO_REQUSICAO_INFOS_OPERADOR_LOGADO);
            }
        );  
            
    };
    
    /**
     * Busca as parametrizações do sistema
     * de acordo com o número da organização
     * do operador logado e seta no templateManager.
    */
    this.setCrmParametrizationInTemplateManager = function(){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        return General.callRestEngine("product#/getCrmParametrization", "DataSet", [{NRORG: loggedOperatorInfos.NRORG}]).then(function(response){
            
            if(!response.dataset.DADOS.PARAMETRIZACAO){
                
                ScreenService.showMessage(Messages.ERRO_REQUSICAO_PARAMETRIZACOES);
                return;
            }
        
            templateManager.CRM_PARAMETRIZACAO = response.dataset.DADOS.PARAMETRIZACAO[0];
        }.bind(this), 
            function(error){
                ScreenService.showMessage(Messages.ERRO_REQUSICAO_PARAMETRIZACOES);
            }
        ); 
      
    };
    
    /**
     * Busca as hierarquias inferiores ao operador
     * logado.
     * 
     * @param {object} LOGGEDOPERATOR objecto com as informações do
     * operador logado.
    */
    this.getInferiorHierarchiesByLoggedOperator = function(LOGGEDOPERATOR) {
        
        var filter = [
            {
                "name": "P_HIERARQUIATUAL",
                "operator": "=",
                "value": LOGGEDOPERATOR.NRHIERARQUIAOPER
            }
        ];
        
        return localStorageEngine.download("product#/crud/crm_hierarquiaoper_hierarquiasinferior", filter).then(function(response){
            this.setLoggedOperatorInferiorHierarchiesInLoggedOperator(response);
        }.bind(this));
        
    };
    
    /**
     * Cria dois arrays, no objeto com as informações do 
     * operador logado, um mostrando os códigos do operadores
     * com hierarquia inferior ao operador logado e outro com o
     * número do parceiro dos operadores.
     * 
     * @param {object} hierarchies operadores com hierarquia
     * abaixo do operador logado.
    */
    this.setLoggedOperatorInferiorHierarchiesInLoggedOperator = function(hierarchies){
        
        var loggedOperator = templateManager.LOGGEDOPERATOR;
        
        loggedOperator.CDOPERSHIERARQINF = [loggedOperator.CDOPERADOR];
        loggedOperator.NRPARCOPERSHIERARQINF = [loggedOperator.NRPARCNEGOCIO];
        
        hierarchies.forEach(function(hierarchy){
            
            loggedOperator.CDOPERSHIERARQINF.push(hierarchy.CDOPERADOR);
            loggedOperator.NRPARCOPERSHIERARQINF.push(hierarchy.NRPARCNEGOCIO);
            
        });
    };
    
    this.setAlterarParametros = function(session) {
        var filtro = [
            {
                "name": "P_NRORG",
                "operator": "=",
                "value": session.user.nrorg
            },
            {
                "name": "P_CDOPERADOR",
                "operator": "=",
                "value": session.user.cdoperador
            }
        ];
    
        General.callRestEngine("product#/getRelatedOrganizations", "FilterData", filtro).then(function(response) {
            if(response.dataset && response.dataset.getRelatedOrganizations && response.dataset.getRelatedOrganizations.length > 1) {
                templateManager.project.userDataActions = [
                    {
                        "label": "Alterar Parâmetros",
                        "icon": "refresh",
                        "events": [
                            {
                                "name": "ActionEvent",
                                "code": "ScreenService.openWindow('product#par3002_paramconfig');ScreenService.toggleSideMenu();"
                            }
                        ]
                    }
                ];
            } else {
                templateManager.project.userDataActions = [];
            }
        });
    
        General.callRestEngine("product#/getNrOrgTrabalhoAtual", "FilterData", filtro).then(function(response) {
            if(!Util.isEmptyOrBlank(response.dataset.getOperador)) {
                var userInfo = response.dataset.getOperador[0].NMORGTRAB + ' - ' + response.dataset.getOperador[0].CRM_NRORGTRAB;
                ScreenService.setHeaderInfo(userInfo);
                ScreenService.buildUserData(templateManager.LOGGEDOPERATOR.NMOPERADOR, [userInfo], function() {}, './assets/images/user.jpg');
                templateManager.LOGGEDOPERATOR.CRM_NRORGTRAB = response.dataset.getOperador[0].CRM_NRORGTRAB;
                templateManager.LOGGEDOPERATOR.NMORGTRAB = response.dataset.getOperador[0].NMORGTRAB;
            }
        });
    };
    
    eventAggregator.subscribe('setParamsProduct', function(session) {
        var user = session.user;

        CRM_Operador_Repository.getNrOrgTrabalhoAtual(user).then(function(data) {
            if(!Util.isEmptyOrBlank(data.dataset.getOperador)) {
                setLocalVar('CRM_NRORGTRABALHO', data.dataset.getOperador[0].CRM_NRORGTRAB);
                setLocalVar('NMORGTRAB', data.dataset.getOperador[0].NMORGTRAB);
            }
        });
        
    });

    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crmLoginController', crmLoginController);
}); 